@@ -1,4 +1,16 @@
 
+if (!isGeneric("RGB2col")) {setGeneric("RGB2col", function(x, ...) standardGeneric("RGB2col"))}
+
+
+setMethod("RGB2col", signature(x="SpatRaster"), 
+	function(x, alpha=FALSE, filename="", overwrite=FALSE, ...) {
+		warn("col2RGB", "this function will be removed. You can use 'colorize' instead")
+		rgb2col(x, alpha=FALSE, filename="", overwrite=FALSE, ...)
+	}
+)
+
+
+
 ## spatstat conflicts
 
 if (!isGeneric("area")) {setGeneric("area", function(x, ...) standardGeneric("area"))}
@@ -9,7 +21,7 @@ setMethod("area", signature(x="SpatRaster"),
 			error("area", 'area(x, sum=FALSE) will be removed. Use "cellSize(x)"')
 		} else {
 			error("area", 'area(x, sum=TRUE) will be removed. Use "expanse(x)" or "global(cellSize(x), "sum")"')
-		}		
+		}
 	}
 )
 
@@ -20,3 +32,4 @@ setMethod("area", signature(x="SpatVector"),
 )
 
 
+
