@@ -31,10 +31,10 @@ from_stars <- function(from) {
 		}
 		return(ra)
 	}
-	
+
 	dims <- attr(from, "dimensions")
 	dd <- dim(from)
-	
+
 	# x, y
 	hasBands <- "band" %in% names(dd)
 	hasTime <- "time" %in% names(dd)
@@ -117,8 +117,8 @@ from_stars <- function(from) {
 	names(s) <- paste(names(dd)[4], 1:length(s), sep="-")
 	s
 }
-	
-	
+
+
 
 setAs("stars", "SpatRasterDataset",
 	function(from) {
@@ -130,7 +130,7 @@ setAs("ggmap", "SpatRaster",
 	function(from) {
 		b <- attr(from, "bb")
 		e <- ext(b$ll.lon, b$ur.lon, b$ll.lat, b$ur.lat)
-		r <- rast(nrows=nrow(from), ncols=ncol(from), ext=e, nlyr=3, crs="epsg:4326")
+		r <- rast(nrows=nrow(from), ncols=ncol(from), ext=e, nlyr=3, crs="epsg:4326", names=c("red", "green", "blue"))
 		values(r) <- t(grDevices::col2rgb(from))
 		RGB(r) <- 1:3
 		r
@@ -138,56 +138,6 @@ setAs("ggmap", "SpatRaster",
 )
 
 
-### from terra
-setAs("SpatRaster", "Raster", 
-	function(from) {
-		s <- sources(from)
-		nl <- nlyr(from)
-		e <- as.vector(ext(from))
-		prj <- crs(from)
-		if (nl == 1) {
-			if (s$source == "") {
-				r <- raster::raster(ncols=ncol(from), nrows=nrow(from), crs=crs(from),
-			          xmn=e[1], xmx=e[2], ymn=e[3], ymx=e[4])
-				if (hasValues(from)) {
-					raster::values(r) <- values(from)
-				}
-			} else {
-				b <- sources(from, TRUE)
-				r <- raster::raster(s$source, band=b$bands)
-			}
-			names(r) <- names(from)
-		} else {
-			b <- sources(from, TRUE)
-			if ((nrow(s) == 1) & (s$source[1] != "")) {
-				r <- raster::brick(s$source)
-				if (!((raster::nlayers(r) == nl) && (b$bands[1] == 1) && (all(diff(b$bands) == 1)))) {
-					r <- raster::stack(s$source, bands=b$bands)
-				}
-			} else if (all(s$source=="")) {
-				r <- raster::brick(ncol=ncol(from), nrow=nrow(from), crs=prj,
-			          xmn=e[1], xmx=e[2], ymn=e[3], ymx=e[4], nl=nlyr(from))
-				if (hasValues(from)) {
-					raster::values(r) <- values(from)
-				}
-			} else {
-				x <- raster::raster(ncol=ncol(from), nrow=nrow(from), crs=prj,
-			          xmn=e[1], xmx=e[2], ymn=e[3], ymx=e[4])
-				r <- list()
-				for (i in 1:nl) {
-					if (s$source[i] == "") {
-						r[[i]] <- raster::setValues(x, values(from[[i]]))
-					} else {
-						bands <- b$bands[b$sid == i]
-						r[[i]] <- raster::stack(s$source[i], bands=bands)
-					}
-				}
-				r <- raster::stack(r)
-			}
-		}
-		return(r)
-	}
-)
 
 
 
@@ -230,7 +180,7 @@ setMethod("as.polygons", signature(x="SpatRaster"),
 	function(x, trunc=TRUE, dissolve=TRUE, values=TRUE, na.rm=TRUE, extent=FALSE) {
 		p <- methods::new("SpatVector")
 		if (extent) {
-			p@ptr <- x@ptr$dense_extent()
+			p@ptr <- x@ptr$dense_extent(FALSE, FALSE)
 			x <- messages(x, "as.polygons")
 		} else {
 			opt <- spatOptions()
@@ -248,10 +198,10 @@ setMethod("as.polygons", signature(x="SpatRaster"),
 						cg <- cgs[[f]]
 						i <- match(unlist(p[[f]]), cg[,1])
 						act <- activeCat(x, f)
-						p[[f]] <- cg[i, act+1]				
+						p[[f]] <- cg[i, act+1]
 					}
 				}
-			}				
+			}
 		}
 		messages(p, "as.polygons")
 	}
@@ -319,7 +269,7 @@ setMethod("as.points", signature(x="SpatRaster"),
 		opt <- spatOptions()
 		p@ptr <- x@ptr$as_points(values, na.rm, opt)
 		x <- messages(x, "as.points")
-			
+
 		if (values) {
 			ff <- is.factor(x)
 			if (any(ff)) {
@@ -332,7 +282,7 @@ setMethod("as.points", signature(x="SpatRaster"),
 					p[[f]] <- as.character(v)
 				}
 			}
-		}	
+		}
 		messages(p, "as.points")
 	}
 )
@@ -363,75 +313,55 @@ setMethod("as.vector", signature(x="SpatRaster"),
 	}
 )
 
-
-setMethod("as.matrix", signature(x="SpatRaster"), 
-	function(x, wide=FALSE) {
-		if (!hasValues(x)) {
-			error("as.matrix", "SpatRaster has no cell values")
-		}
-		if (wide) {
-			if (nlyr(x) > 1) {
-				m <- values(x, mat=TRUE)
-				m <- lapply(1:ncol(m), function(i) {
-					matrix(m[,i], nrow=nrow(x), byrow=TRUE)
-					})
-				m <- do.call(cbind, m)
-			} else {
-				m <- matrix(values(x, mat=FALSE),nrow=nrow(x),byrow=TRUE)
-			}
-		} else {
+as.matrix.SpatRaster <- function(x, ...) {
+	if (!hasValues(x)) {
+		error("as.matrix", "SpatRaster has no cell values")
+	}
+	wide <- isTRUE(list(...)$wide)
+	if (wide) {
+		if (nlyr(x) > 1) {
 			m <- values(x, mat=TRUE)
-		}
-		m
-	}
-)
-
-
-setMethod("as.data.frame", signature(x="SpatRaster"), 
-	function(x, xy=FALSE, cells=FALSE, na.rm=TRUE) {
-		d <- NULL
-		if (xy) {
-			d <- xyFromCell(x, 1:ncell(x))
-		} 
-		if (cells) {
-			d <- cbind(cell=1:ncell(x), d)
-		}
-		if (is.null(d)) {
-			d <- values(x, dataframe=TRUE)
+			m <- lapply(1:ncol(m), function(i) {
+				matrix(m[,i], nrow=nrow(x), byrow=TRUE)
+				})
+			m <- do.call(cbind, m)
 		} else {
-			d <- data.frame(d, values(x, dataframe=TRUE))
+			m <- matrix(values(x, mat=FALSE),nrow=nrow(x),byrow=TRUE)
 		}
-		if (na.rm) {
-			d <- stats::na.omit(d) 
-			attr(d, "na.action") <- NULL
-		}
-		d
+	} else {
+		m <- values(x, mat=TRUE)
 	}
-)
+	m
+}
+setMethod("as.matrix", signature(x="SpatRaster"), as.matrix.SpatRaster)
 
 
-if (!isGeneric("as.data.table")) { setGeneric("as.data.table", function(x, ...) standardGeneric("as.data.table")) }	
+as.data.frame.SpatRaster <- function(x, row.names=NULL, optional=FALSE, xy=FALSE, cells=FALSE, na.rm=TRUE, ...) {
+#	dots <- list(...) 
+#	xy <- isTRUE(dots$xy) 
+#	cells <- isTRUE(dots$cells)
+#	na.rm <- isTRUE(dots$na.rm)
 
-setMethod("as.data.table", signature(x="SpatRaster"), 
-	function(x, xy=FALSE, cells=FALSE, na.rm=TRUE) {
-		d <- data.table::data.table()
-		if (xy) {
-			d <- cbind(d, xyFromCell(x, 1:ncell(x)))
-		} 
-		if (cells) {
-			d <- cbind(cell=1:ncell(x), d)
-		}
-		if (any(is.factor(x))) {
-			d <- cbind(d, values(x, dataframe=TRUE))
-		} else {
-			d <- cbind(d, values(x, dataframe=FALSE))
-		}
-		if (na.rm) {
-			d <- stats::na.omit(d) 
-		}
-		d
+	d <- NULL
+	if (xy) {
+		d <- xyFromCell(x, 1:ncell(x))
+	} 
+	if (cells) {
+		d <- cbind(cell=1:ncell(x), d)
 	}
-)
+	if (is.null(d)) {
+		d <- values(x, dataframe=TRUE, ... )
+	} else {
+		d <- data.frame(d, values(x, dataframe=TRUE), ...)
+	}
+	if (na.rm) {
+		d <- stats::na.omit(d) 
+		attr(d, "na.action") <- NULL
+	}
+	d
+}
+setMethod("as.data.frame", signature(x="SpatRaster"), as.data.frame.SpatRaster)
+
 
 
 setAs("SpatRaster", "data.frame", 
@@ -459,88 +389,6 @@ setMethod("as.array", signature(x="SpatRaster"),
 )
 
 
-
-.fromRasterLayerBrick <- function(from) {
-	 
-	if (raster::fromDisk(from)) {
-		f <- raster::filename(from)
-		if (from@file@driver == "netcdf") {
-			v <- attr(from@data, "zvar")
-			r <- rast(f, v)	
-		} else {
-			r <- try(rast(f), silent=TRUE)
-			if (inherits(r, "try-error")) {
-				r <- rast(from + 0)
-				levs <- levels(from)[[1]]
-				if (!is.null(levs)) {
-					levels(r) <- levs
-				}
-			}
-			crs(r) <- raster::wkt(from)
-		}
-		if (from@file@NAchanged) {
-			NAflag(r) <- from@file@nodatavalue
-		}
-		return(r)
-	} else {
-		crsobj <- from@crs
-		if (is.na(crsobj)) {
-			prj <- ""
-		} else {
-			crscom <- comment(crsobj)
-			if (is.null(crscom)) {
-				prj <- crsobj@projargs
-			} else {
-				prj <- crscom
-			}
-		}
-		r <- rast(	nrows=nrow(from), 
-					ncols=ncol(from),
-					nlyrs=raster::nlayers(from),
-					crs=prj,
-					extent=raster::extent(from))
-		if (raster::hasValues(from)) {
-			values(r) <- raster::values(from)
-		}
-		names(r)  <- names(from)
-		levs <- raster::levels(from)[[1]]
-		if (!is.null(levs)) {
-			levels(r) <- levs				
-		}
-	}
-	r
-}
-
-.fromRasterStack <- function(from) {
-	x <- from[[1]]
-	n <- raster::nbands(x)
-	nl <- raster::nlayers(from)
-	if ((n > 1) & (n == nl)) {
-		ff <- lapply(1:nl, function(i) { raster::filename(from[[i]]) })
-		if (length(unique(ff)) == 1) {
-			r <- rast(raster::filename(x))
-			return(r)
-		}
-	} 
-	s <- lapply(1:raster::nlayers(from), function(i) {
-		x <- from[[i]]
-		.fromRasterLayerBrick(x)[[raster::bandnr(x)]]
-	})
-	do.call(c, s)
-}
-
-
-setAs("Raster", "SpatRaster", 
-	function(from) {
-		if (inherits(from, "RasterLayer") || inherits(from, "RasterBrick")) { 
-			.fromRasterLayerBrick(from)
-		} else {
-			.fromRasterStack(from)
-		}
-	}
-)
-
-
 # to sf from SpatVector
 # available in sf
 #.v2sf <- function(from) {
@@ -574,124 +422,6 @@ setAs("Raster", "SpatRaster",
 }
 
 
-...from_sf <- function(from) {
-	sfi <- attr(from, "sf_column")
-	geom <- from[[sfi]]
-	crs <- attr(geom, "crs")$wkt
-	attr(geom, "class") <- NULL
-	types <- t(sapply(geom, function(i) attr(i, "class")))
-	v <- list()
-	for (i in 1:length(geom)) {
-		if (inherits(geom[[i]], "POINT")) {
-			v[[i]] <- cbind(i, 1, geom[[i]][1], geom[[i]][2], hole=0)				
-		} else {
-			vv <- list()
-			for (j in 1:length(geom[[i]])) {
-				if (inherits(geom[[i]][[j]], "list")) {
-					vvv <- list()
-					for (k in 1:length(geom[[i]][[j]])) {
-						vvv[[k]] <- cbind(i, j, geom[[i]][[j]][[k]], hole=k-1) 
-					}
-					vv[[j]] <- do.call(rbind, vvv)
-				} else {
-					vv[[j]] <- cbind(i, j, geom[[i]][[j]], hole=0)
-				}
-			}
-			v[[i]] <- do.call(rbind, vv)
-		}
-	}
-	v <- do.call(rbind, v)
-	if (ncol(v) > 5) {
-		v <- cbind(v[,1:4], v[,ncol(v),drop=FALSE])
-		warn("as", "Z/M dimension dropped")
-	}
-	colnames(v)[1:4] <- c("id", "part", "x", "y")
-	types <- unique(gsub("MULTI", "", unique(types[,2])))
-	if (length(types) > 1) {
-		error("as,sf", "SpatVector currently only accepts one geometry type")
-	}
-	if (grepl("POINT", types, fixed=TRUE)) {
-		gt = "points"
-	} else if (grepl("LINE", types, fixed=TRUE)) {
-		gt = "lines"
-	} else if (grepl("POLY", types, fixed=TRUE)) {
-		gt = "polygons"
-	}
-	if (ncol(from) > 1) {
-		from[[sfi]] <- NULL
-		d <- as.data.frame(from)
-		vect(v, type=gt, att=d, crs=crs)
-	} else {
-		vect(v, type=gt, crs=crs)
-	}
-}
-
-
-
-
-...from_sfc <- function(from) {
-	geom = from
-	v <- list()
-	for (i in 1:length(geom)) {
-		vv <- list()
-		for (j in 1:length(geom[[i]])) {
-			if (inherits(geom[[i]][[j]], "list")) {
-				vvv <- list()
-				for (k in 1:length(geom[[i]][[j]])) {
-					vvv[[k]] <- cbind(i, j, geom[[i]][[j]][[k]], hole= k-1) 
-				}
-				vv[[j]] <- do.call(rbind, vvv)
-			} else {
-				vv[[j]] <- cbind(i, j, geom[[i]][[j]], hole=0) 
-			}
-		}
-		v[[i]] <- do.call(rbind, vv)
-	}
-	v <- do.call(rbind, v)
-	colnames(v)[1:4] <- c("id", "part", "x", "y")
-	if (ncol(v) > 5) {
-		v <- cbind(v[,1:4], v[,ncol(v),drop=FALSE])
-		warn("as", "Z/M dimension dropped")
-	}
-	types <- class(from)[1]
-	if (grepl("POINT", types, fixed=TRUE)) {
-		gt = "points"
-	} else if (grepl("LINE", types, fixed=TRUE)) {
-		gt = "lines"
-	} else if (grepl("POLY", types, fixed=TRUE)) {
-		gt = "polygons"
-	}
-	vect(v, type=gt, crs="")
-}
-
-
-...from_sfg <- function(from) {
-	geom = from
-	v <- list()
-	for (i in 1:length(geom)) {
-		vv <- list()
-		for (j in 1:length(geom[[i]])) {
-			vv[[j]] <- cbind(i, j, geom[[i]][[j]], hole= j-1) 
-		}
-		v[[i]] <- do.call(rbind, vv)
-	}
-	v <- do.call(rbind, v)
-	colnames(v)[1:4] <- c("id", "part", "x", "y")
-	if (ncol(v) == 6) {
-		v <- v[,-5]
-	}
-	types <- class(geom)[2]
-	if (grepl("POINT", types, fixed=TRUE)) {
-		gt = "points"
-	} else if (grepl("LINE", types, fixed=TRUE)) {
-		gt = "lines"
-	} else if (grepl("POLY", types, fixed=TRUE)) {
-		gt = "polygons"
-	}
-	vect(v, type=gt, crs="")
-}
-
-
 setAs("sf", "SpatVector", 
 	function(from) {
 		v <- try(.from_sf(from), silent=TRUE)
@@ -705,6 +435,7 @@ setAs("sf", "SpatVector",
 .from_sfc <- function(from) {
 	v <- vect()
 	v@ptr <- v@ptr$from_hex(sf::rawToHex(sf::st_as_binary(from)), "")
+	crs(v) <- attr(from, "crs")$wkt
 	v
 }
 
@@ -752,14 +483,14 @@ setAs("im", "SpatRaster",
 setAs("SpatVector", "Spatial", 
 	function(from) {
 		g <- geom(from, df=TRUE)
-		raster::geom(g, values(from), geomtype(from), crs(from))
+		geom(g, values(from), geomtype(from), crs(from))
 	}
 )
 
 
 setAs("Spatial", "SpatVector", 
 	function(from) {
-		g <- raster::geom(from, df=TRUE)
+		g <- geom(from, df=TRUE)
 		colnames(g)[1] <- "id"
 		if (inherits(from, "SpatialPolygons")) {
 			vtype <- "polygons"
@@ -776,9 +507,9 @@ setAs("Spatial", "SpatVector",
 			g <- cbind(g[,1,drop=FALSE], part=1:nrow(g), g[,2:3,drop=FALSE])
 		}
 		if (methods::.hasSlot(from, "data")) {
-			v <- vect(g, vtype, from@data, raster::crs(from))
+			v <- vect(g, vtype, from@data, crs(from))
 		} else {
-			v <- vect(g, vtype, crs=raster::crs(from))
+			v <- vect(g, vtype, crs=crs(from))
 		}
 		return(v)
 	}
