@@ -6,6 +6,7 @@ setMethod ("coltab" , "SpatRaster",
 		if (any(hascols)) {
 			d <- x@ptr$getColors()
 			d <- lapply(d, .getSpatDF)
+			d[!hascols] <- list(NULL)
 		} else {
 			d <- vector("list", length(hascols))
 		}
@@ -16,13 +17,13 @@ setMethod ("coltab" , "SpatRaster",
 
 setMethod ("coltab<-" , "SpatRaster", 
 	function(x, layer=1, value) {
-		stopifnot(hasValues(x))
+		#stopifnot(hasValues(x))
 		if (missing(value)) {
 			value <- layer
 			layer <- 1
 		}
 		layer <- layer[1]-1
-		
+
 		if (is.null(value)) {
 			x@ptr$removeColors(layer)
 			return(x)
@@ -41,11 +42,16 @@ setMethod ("coltab<-" , "SpatRaster",
 		}
 
 		stopifnot(inherits(value, "data.frame"))
-
-		value <- value[1:256,]
-		value[is.na(value)] <- 255
-		value <- data.frame(sapply(value, function(i) as.integer(clamp(i, 0, 255))))
-		if (ncol(value) == 3) {
+		nms <- tolower(names(value))
+		if (!("value" %in% nms)) {
+			value <- cbind(values=(1:nrow(value))-1, value)
+		}
+		#value <- value[1:256,]
+		for (i in 2:ncol(value)) {
+			value[is.na(value), i] <- 255
+			value[, i] <- as.integer(clamp(value[, i], 0, 255))
+		} 
+		if (ncol(value) == 4) {
 			value <- cbind(value, alpha=255)
 		}
 
