@@ -10,7 +10,7 @@ function(x, mx=10000, ...) {
 )
 
 setMethod("lines", signature(x="SpatVector"), 
-	function(x, y=NULL, col, lwd=1, lty=1, arrows=FALSE, ...)  {
+	function(x, y=NULL, col, lwd=1, lty=1, arrows=FALSE, alpha=1, ...)  {
 		gtype <- geomtype(x)
 		if (missing(col)) col <- "black"
 		if (!is.null(y)) {
@@ -19,7 +19,7 @@ setMethod("lines", signature(x="SpatVector"),
 			if ((ytype != "points") || (gtype != "points")) {
 				error("lines", "when supplying two SpatVectors, both must have point geometry")
 			}
-			stopifnot(nrow(x) == nrow(y))	
+			stopifnot(nrow(x) == nrow(y))
 			p1 <- geom(x)[, c("x", "y"), drop=FALSE]
 			p2 <- geom(y)[, c("x", "y"), drop=FALSE]
 			if (arrows) {
@@ -27,13 +27,13 @@ setMethod("lines", signature(x="SpatVector"),
 			} else {
 				a <- as.vector(t(cbind(p1[,1], p2[,1], NA)))
 				b <- as.vector(t(cbind(p1[,2], p2[,2], NA)))
-				lines(cbind(a, b), col=col, lwd=lwd, lty=lty, ...)
+				lines(cbind(a, b), col=col, lwd=lwd, lty=lty, alpha=alpha, ...)
 			}
 		} else if (grepl("points", gtype)) {
-			points(x, col=col, type="l", lwd=lwd, lty=lty, ...)
+			points(x, col=col, type="l", lwd=lwd, lty=lty, alpha=alpha, ...)
 		} else {
 			n <- length(x)
-			col <- .getCols(n, col)
+			col <- .getCols(n, col, alpha)
 			lwd <- rep_len(lwd, n)
 			lty <- rep_len(lty, n)
 			g <- geom(x, df=TRUE)
@@ -56,10 +56,10 @@ setMethod("lines", signature(x="SpatVector"),
 
 
 setMethod("points", signature(x="SpatVector"), 
-	function(x, col, cex=1, pch=20, ...)  {
+	function(x, col, cex=1, pch=20, alpha=1, ...)  {
 		if (missing(col)) col <- "black"
 		n <- length(x)
-		col <- .getCols(n, col)
+		col <- .getCols(n, col, alpha)
 		cex <- rep_len(cex, n)
 		pch <- rep_len(pch, n)
 		g <- geom(x, df=TRUE)
@@ -76,7 +76,7 @@ setMethod("points", signature(x="SpatVector"),
 
 
 setMethod("polys", signature(x="SpatVector"), 
-	function(x, col, border="black", lwd=1, lty=1, ...)  {
+	function(x, col, border="black", lwd=1, lty=1, alpha=1, ...)  {
 		gtype <- geomtype(x)
 		if (gtype != "polygons") {
 			error("polys", "expecting polygons")
@@ -84,7 +84,7 @@ setMethod("polys", signature(x="SpatVector"),
 		if (missing(col)) {
 			col <- NULL
 		}
-		cols <- .getCols(length(x), col)
+		cols <- .getCols(length(x), col, alpha)
 		out <- list(main_cols=cols)
 		out$leg$border <- border
 		.plotPolygons(x, out, lwd=lwd, lty=lty, ...)
