@@ -4,11 +4,17 @@
 # License GPL v3
 
 setMethod("lines", signature(x="SpatExtent"), 
-	function(x, col, ...)  {
+	function(x, col="black", alpha=1, ...)  {
 		e <- as.vector(x)
 		p <- rbind(c(e[1],e[3]), c(e[1],e[4]), c(e[2],e[4]), c(e[2],e[3]), c(e[1],e[3]))
-		if (missing(col)) col <- "black"
-		graphics::lines(p, col=col[1], ...)
+		col <- .getCols(1, col, alpha)
+		graphics::lines(p, col=col, ...)
+	}
+)
+
+setMethod("polys", signature(x="SpatExtent"), 
+	function(x, col="black", alpha=1, ...)  {
+		polys(as.polygons(x), col=col, alpha=alpha, ...)
 	}
 )
 
@@ -23,12 +29,10 @@ setMethod("plot", signature(x="SpatExtent", y="missing"),
 )
 
 setMethod("points", signature(x="SpatExtent"), 
-	function(x, col, ...)  {
+	function(x, col="black", alpha=1, ...)  {
 		e <- as.vector(x)
 		p <- rbind(c(e[1],e[3]), c(e[1],e[4]), c(e[2],e[4]), c(e[2],e[3]), c(e[1],e[3]))
-		if (missing(col)) col <- "black"
-		col <- .getCols(4, col)
-		if (is.null(col)) col <- "black"
+		col <- .getCols(4, col, alpha)
 		graphics::points(p, col=col, ...)
 	}
 )
