@@ -121,7 +121,7 @@ setMethod("dots", signature(x="SpatVector"),
 }
 
 
-.getCols <- function(n, cols) {
+.getCols <- function(n, cols, alpha=1) {
 	if (!is.null(cols)) {
 		ncols <- length(cols)
 		if (ncols > n) {
@@ -132,13 +132,15 @@ setMethod("dots", signature(x="SpatVector"),
 			cols <- rep_len(cols, n)
 		}
 	} 
+	if (alpha < 1 && alpha >= 0) {
+		cols <- grDevices::rgb(t(grDevices::col2rgb(cols)), alpha=alpha[1]*255, maxColorValue=255)
+	}
 	cols
 }
 
-
 .vect.legend.none <- function(out) {
 	#if (out$leg$geomtype == "points") {
-		out$main_cols <- .getCols(out$ngeom, out$cols)
+		out$main_cols <- .getCols(out$ngeom, out$cols, 1)
 	#} else {
 	#	out$cols <- .getCols(out$ngeom, out$cols)
 	#}
@@ -146,14 +148,20 @@ setMethod("dots", signature(x="SpatVector"),
 }
 
 .vect.legend.classes <- function(out) {
-	ucols <- .getCols(length(out$uv), out$cols)
 
-	out$uv <- sort(out$uv)
+	if (isTRUE(out$legend_sort)) {
+		out$uv <- sort(out$uv)
+	}
+	ucols <- .getCols(length(out$uv), out$cols, 1)
 
 	i <- match(out$v, out$uv)
 	out$cols <- ucols
 	out$main_cols <- ucols[i]
 
+	if (!is.null(out$colNA)) {
+		out$main_cols[is.na(out$main_cols)] <- out$colNA
+	}
+	
 	out$levels <- out$uv
 	out$leg$legend <- out$uv
 	nlevs <- length(out$uv)
@@ -173,6 +181,7 @@ setMethod("dots", signature(x="SpatVector"),
 		out$leg$x <- "top"
 	}
 
+
 	out
 }
 
@@ -265,21 +274,30 @@ setMethod("dots", signature(x="SpatVector"),
 	}
 
 	out$main_cols <- out$cols[out$vcut]
+	
+	if (!is.null(out$colNA)) {
+		out$main_cols[is.na(out$main_cols)] <- out$colNA
+	}
 	out
 }
 
 
 
-.plot.vect.map <- function(x, out, xlab="", ylab="", type = "n", yaxs="i", xaxs="i", asp=out$asp, density=NULL, angle=45, border="black", dig.lab=3, main="", ...) {
+.plot.vect.map <- function(x, out, xlab="", ylab="", type = "n", yaxs="i", xaxs="i", asp=out$asp, density=NULL, angle=45, border="black", dig.lab=3, main="", sort=TRUE, ...) {
 
 	if ((!out$add) & (!out$legend_only)) {
 		if (!any(is.na(out$mar))) { graphics::par(mar=out$mar) }
 		plot(out$lim[1:2], out$lim[3:4], type="n", xlab=xlab, ylab=ylab, asp=asp, xaxs=xaxs, yaxs=yaxs, axes=FALSE, main=main)
+		if (!is.null(out$background)) {
+			usr <- graphics::par("usr")
+			graphics::rect(usr[1], usr[3], usr[2], usr[4], col=out$background)
+		}
 	}
 
 	out$leg$density <- density
 	out$leg$angle <- angle
 	out$leg$border <- border
+	out$legend_sort <- isTRUE(sort)
 
 	nuq <- length(out$uv)
 	if (out$legend_type == "none") {
@@ -329,34 +347,43 @@ setMethod("dots", signature(x="SpatVector"),
 
 .prep.vect.data <- function(x, y, type, cols=NULL, mar=NULL, legend=TRUE, 
 	legend.only=FALSE, levels=NULL, add=FALSE, range=NULL, breaks=NULL, breakby="eqint",
-	xlim=NULL, ylim=NULL, colNA=NA, alpha=NULL, axes=TRUE, main=NULL,  
-	pax=list(), plg=list(), ...) {
+	xlim=NULL, ylim=NULL, colNA=NA, alpha=NULL, axes=TRUE, main=NULL, buffer=TRUE, background=NULL,
+	pax=list(), plg=list(), ext=NULL, grid=FALSE, las=0, ...) {
 
 	out <- list()
 	out$ngeom <- nrow(x)
+
 	e <- as.vector(ext(x))
 	out$ext <- e
-
-	if (!is.null(xlim)) {
-		stopifnot(length(xlim) == 2)
-		e[1:2] <- sort(xlim)
+	if (!is.null(ext)) {
+		stopifnot(inherits(ext, "SpatExtent"))
+		x <- crop(x, ext)
+		out$ext <- as.vector(ext(x))
+		out$lim <- ext
 	} else {
-		dx <- diff(e[1:2]) / 50
-		e[1:2] <- e[1:2] + c(-dx, dx)
+		if (!is.null(xlim)) {
+			stopifnot(length(xlim) == 2)
+			e[1:2] <- sort(xlim)
+		} else if (buffer) {
+			dx <- diff(e[1:2]) / 50
+			e[1:2] <- e[1:2] + c(-dx, dx)
+		}
+		if (!is.null(ylim)) {
+			stopifnot(length(ylim) == 2)
+			e[3:4] <- sort(ylim)
+		} else if (buffer) {
+			dy <- diff(e[3:4]) / 50
+			e[3:4] <- e[3:4] + c(-dy, dy)
+		}
+		out$lim <- e
 	}
-	if (!is.null(ylim)) {
-		stopifnot(length(ylim) == 2)
-		e[3:4] <- sort(ylim)
-	} else {
-		dy <- diff(e[3:4]) / 50
-		e[3:4] <- e[3:4] + c(-dy, dy)
-	}
-	out$lim <- e
-
+	
 	out$add <- isTRUE(add)
 	out$axes <- isTRUE(axes)
-	out$axs <- pax 
-	out$leg <- plg
+	out$axs <- as.list(pax)
+	if (is.null(out$axs$las)) out$axs$las <- las
+	out$draw_grid <- isTRUE(grid)	
+	out$leg <- as.list(plg)
 	out$leg$geomtype <- geomtype(x)
 	out$asp <- 1
 	out$lonlat <- is.lonlat(x, perhaps=TRUE, warn=FALSE)
@@ -365,7 +392,7 @@ setMethod("dots", signature(x="SpatVector"),
 	}
 	out$breaks <- breaks
 	out$breakby <- breakby
-
+	out$background <- background
 	v <- unlist(x[, y, drop=TRUE], use.names=FALSE)
 	if (!is.null(range)) {
 		range <- sort(range)
@@ -430,10 +457,14 @@ setMethod("dots", signature(x="SpatVector"),
 	}
 	out$mar <- rep_len(mar, 4)
 
+	out$skipNA <- TRUE
 	if (!is.null(colNA)) {
 		if (!is.na(colNA)) {
 			out$colNA <- grDevices::rgb(t(grDevices::col2rgb(colNA)), alpha=alpha, maxColorValue=255)
 			out$r[is.na(out$r)] <- out$colNA
+			out$skipNA <- FALSE
+ 		} else {
+			out$colNA <- NULL
 		}
 	}
 
@@ -443,7 +474,8 @@ setMethod("dots", signature(x="SpatVector"),
 
 setMethod("plot", signature(x="SpatVector", y="character"), 
 	function(x, y, col=NULL, type, mar=NULL, legend=TRUE, add=FALSE, axes=!add, 
-	main=y, plg=list(), pax=list(), nr, nc, ...) {
+	main=y, buffer=TRUE, background=NULL, grid=FALSE, ext=NULL,
+	plg=list(), pax=list(), nr, nc, ...) {
 
 		if (nrow(x) == 0) {
 			error("plot", "SpatVector has zero geometries")
@@ -454,13 +486,13 @@ setMethod("plot", signature(x="SpatVector", y="character"),
 			i <- is.na(match(y, names(x)))
 			error("plot", paste(paste(y[i], collapse=",")), " is not a name in x")
 		}
-		nrnc <- c(1,1)
-		if (length(y) > 1) {
-			nrnc <- .get_nrnc(nr, nc, length(y))
-			old.par <- graphics::par(no.readonly =TRUE)
-			on.exit(graphics::par(old.par))   
-			graphics::par(mfrow=nrnc)
-		}
+			nrnc <- c(1,1)
+			if (length(y) > 1) {
+				nrnc <- .get_nrnc(nr, nc, length(y))
+				old.par <- graphics::par(no.readonly =TRUE)
+				on.exit(graphics::par(old.par))   
+				graphics::par(mfrow=nrnc)
+			}
 		if (is.character(legend)) {
 			plg$x <- legend
 			legend <- TRUE
@@ -485,12 +517,12 @@ setMethod("plot", signature(x="SpatVector", y="character"),
 			if (missing(col)) col <- NULL
 
 			if (y[i] == "") {
-				out <- .prep.vect.data(x, y="", type="none", cols=col, mar=mar, plg=list(), pax=pax, legend=FALSE, add=add, axes=axes, main=main[i], ...)
+				out <- .prep.vect.data(x, y="", type="none", cols=col, mar=mar, plg=list(), pax=pax, legend=FALSE, add=add, axes=axes, main=main[i], buffer=buffer, background=background, grid=grid, ext=ext, ...)
 			} else {
-				out <- .prep.vect.data(x, y[i], type=type, cols=col, mar=mar, plg=plg, pax=pax, legend=isTRUE(legend), add=add, axes=axes, main=main[i], ...)
+				out <- .prep.vect.data(x, y[i], type=type, cols=col, mar=mar, plg=plg, pax=pax, legend=isTRUE(legend), add=add, axes=axes, main=main[i], buffer=buffer, background=background, grid=grid, ext=ext, ...)
 			}
-			invisible(out)
 		}
+		invisible(out)		
 	}
 )
 
