@@ -4,7 +4,7 @@
 #include "recycle.h"
 #include "string_utils.h"
 
-	
+
 SpatVector SpatVector::allerretour() {
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
@@ -28,7 +28,7 @@ std::vector<std::string> SpatVector::wkt() {
 	std::vector<std::string> out;
 	out.reserve(g.size());
 	char * wkt;
-	for (size_t i = 0; i < g.size(); i++) {	
+	for (size_t i = 0; i < g.size(); i++) {
 		wkt = GEOSGeomToWKT_r(hGEOSCtxt, g[i].get());
 		out.push_back(wkt);
 	}
@@ -44,7 +44,7 @@ std::vector<std::string> SpatVector::wkb() {
 	std::vector<std::string> out;
 	out.reserve(g.size());
 	size_t len = 0;
-	for (size_t i = 0; i < g.size(); i++) {	
+	for (size_t i = 0; i < g.size(); i++) {
 		unsigned char *wkb = GEOSGeomToWKB_buf_r(hGEOSCtxt, g[i].get(), &len);
 		std::string s( reinterpret_cast<char const*>(wkb), len) ;
 		out.push_back(s);
@@ -60,9 +60,9 @@ std::vector<std::string> SpatVector::hex() {
 	std::vector<std::string> out;
 	out.reserve(g.size());
 	size_t len = 0;
-	for (size_t i = 0; i < g.size(); i++) {	
+	for (size_t i = 0; i < g.size(); i++) {
 		unsigned char *hex = GEOSGeomToHEX_buf_r(hGEOSCtxt, g[i].get(), &len);
-		std::string s( reinterpret_cast<char const*>(hex), len) ;		
+		std::string s( reinterpret_cast<char const*>(hex), len) ;
 		out.push_back(s);
 		free(hex);
 	}
@@ -78,7 +78,7 @@ SpatVector SpatVector::from_hex(std::vector<std::string> x, std::string srs) {
 	std::vector<GeomPtr> p;
 	p.resize(n);
 	for (size_t i = 0; i < n; i++) {
-		const char* cstr = x[i].c_str();	
+		const char* cstr = x[i].c_str();
 		size_t len = strlen(cstr);
 		const unsigned char *hex = (const unsigned char *) cstr;
 		GEOSGeometry* r = GEOSGeomFromHEX_buf_r(hGEOSCtxt, hex, len);
@@ -98,7 +98,7 @@ std::vector<bool> SpatVector::geos_isvalid() {
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
 	std::vector<bool> out;
 	out.reserve(g.size());
-	for (size_t i = 0; i < g.size(); i++) {	
+	for (size_t i = 0; i < g.size(); i++) {
 		char v = GEOSisValid_r(hGEOSCtxt, g[i].get());
 		out.push_back(v);
 	}
@@ -111,7 +111,7 @@ std::vector<std::string> SpatVector::geos_isvalid_msg() {
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
 	std::vector<std::string> out;
 	out.reserve(2 * g.size());
-	for (size_t i = 0; i < g.size(); i++) {	
+	for (size_t i = 0; i < g.size(); i++) {
 		char v = GEOSisValid_r(hGEOSCtxt, g[i].get());
 		std::string valid = {v};
 		out.push_back(valid);
@@ -129,15 +129,55 @@ std::vector<std::string> SpatVector::geos_isvalid_msg() {
 }
 
 
+SpatVector SpatVector::make_valid2() {
+
+	SpatVector out;
+#ifndef GEOS380
+	out.setError("make_valid is not available for GEOS < 3.8");
+#else 
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
+
+	size_t n = size();
+	std::vector<long> ids;
+	ids.reserve(n);
+
+	for (size_t i=0; i<n; i++) {
+		GEOSGeometry* r = GEOSMakeValid_r(hGEOSCtxt, x[i].get());
+		if (r != NULL) {
+			if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
+				x[i] = geos_ptr(r, hGEOSCtxt);
+				ids.push_back(i);
+			} else {
+				GEOSGeom_destroy_r(hGEOSCtxt, r);
+			}
+		}
+	}
+	SpatVectorCollection coll = coll_from_geos(x, hGEOSCtxt, ids, false, false);
+	out = coll.get(0);
+	geos_finish(hGEOSCtxt);
+	out.srs = srs;
+	if (ids.size() != n) {
+		out.df = df.subset_rows(out.df.iv[0]);
+	} else {
+		out.df = df;
+	}
+#endif	
+	return out;
+}
+
+
+
+
 SpatVector SpatVector::crop(SpatExtent e) {
 
 	SpatVector out;
-	
-#ifndef HAVE350
+
+#ifndef GEOS350
 	out.setError("GEOS 3.5 required for crop");
 	return out;
 #else 
-		
+
 	out.srs = srs;
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 
@@ -172,6 +212,8 @@ SpatVector SpatVector::crop(SpatExtent e) {
 }
 
 
+
+
 SpatVector SpatVector::make_nodes() {
 
 	SpatVector out;
@@ -264,7 +306,7 @@ SpatVector SpatVector::normalize() {
 
 
 SpatVector SpatVector::line_merge() {
-		
+
 	SpatVector out;
 	if (type() != "lines") {
 		out.setError("input must be lines");
@@ -311,7 +353,7 @@ SpatVector SpatVector::simplify(double tolerance, bool preserveTopology) {
 	for (size_t i = 0; i < g.size(); i++) {
 		GEOSGeometry* r;
 		if (preserveTopology) {
-			r = GEOSTopologyPreserveSimplify_r(hGEOSCtxt, g[i].get(), tolerance);			
+			r = GEOSTopologyPreserveSimplify_r(hGEOSCtxt, g[i].get(), tolerance);
 		} else {
 			r = GEOSSimplify_r(hGEOSCtxt, g[i].get(), tolerance);
 		}
@@ -339,12 +381,12 @@ SpatVector SpatVector::simplify(double tolerance, bool preserveTopology) {
 
 
 SpatVector SpatVector::shared_paths() {
-	
+
 	if (type() == "polygons") {
 		SpatVector x = as_lines();
 		return x.shared_paths();
 	}
-	
+
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 
@@ -358,14 +400,14 @@ SpatVector SpatVector::shared_paths() {
 				if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
 					p.push_back(geos_ptr(r, hGEOSCtxt));
 					id1.push_back(i+1);
-					id2.push_back(j+1);				
+					id2.push_back(j+1);
 				} else {
 					GEOSGeom_destroy_r(hGEOSCtxt, r);
 				}
 			}
 		}
 	}
-	
+
 	SpatVector out;
 	if (p.size() > 0) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt, std::vector<long>(), false, false);
@@ -389,11 +431,11 @@ SpatVector SpatVector::split_polygons(SpatVector lns) {
 		std::vector<double> yln = {-91, 91};
 		glns.setPart(SpatPart(xln, yln), 1);
 		SpatVector v;
-		v.addGeom(glns);	
+		v.addGeom(glns);
 		v = v.line_merge();
 		v = v.aggregate(false);
 		v = v.polygonize();
-		g = v.geoms[0];		
+		g = v.geoms[0];
 */
 
 SpatVector SpatVector::polygonize() {
@@ -407,7 +449,7 @@ SpatVector SpatVector::polygonize() {
 	p.reserve(g.size());
 	size_t ngeoms = 1;
 	for (size_t i = 0; i < g.size(); i++) {
-		const GEOSGeometry* gi = g[i].get();		
+		const GEOSGeometry* gi = g[i].get();
 		GEOSGeometry* r = GEOSPolygonize_r(hGEOSCtxt, &gi, ngeoms);
 		if (r == NULL) {
 			out.setError("something bad happened");
@@ -438,14 +480,14 @@ SpatVector SpatVector::snap(double tolerance) {
 	if (s == 0) {
 		return out;
 	}
-	
+
 	tolerance = std::max(0.0, tolerance);
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 
 	std::vector<long> ids;
 	ids.reserve(s);
-	
+
 	for (size_t i=0; i<(s-1); i++) {
 		GEOSGeometry* r = x[i].get();
 		for (size_t j=(i+1); j<s; j++) {
@@ -477,12 +519,12 @@ SpatVector SpatVector::snapto(SpatVector y, double tolerance) {
 
 	y = y.aggregate(false);
 	size_t s = size();
-	
+
 	SpatVector out;
 	if (s == 0) {
 		return out;
 	}
-	
+
 	tolerance = std::max(0.0, tolerance);
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
@@ -490,7 +532,7 @@ SpatVector SpatVector::snapto(SpatVector y, double tolerance) {
 
 	std::vector<long> ids;
 	ids.reserve(s);
-	
+
 	GEOSGeometry* gto = to[0].get();
 	for (size_t i=0; i<s; i++) {
 		GEOSGeometry* r = GEOSSnap_r(hGEOSCtxt, x[i].get(), gto, tolerance);
@@ -542,7 +584,7 @@ SpatVector SpatVector::crop(SpatVector v) {
 	std::vector<long> ids;
 	size_t nx = size();
 	ids.reserve(nx);
-	
+
 	for (size_t i = 0; i < nx; i++) {
 		GEOSGeometry* geom = GEOSIntersection_r(hGEOSCtxt, x[i].get(), y[0].get());
 		if (geom == NULL) {
@@ -581,7 +623,7 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 	if (by != "") {
 		SpatVector tmp = aggregate(by, false);
 		if (tmp.hasError()) {
-			return tmp;	
+			return tmp;
 		}
 		for (size_t i=0; i<tmp.size(); i++) {
 			SpatVector x = tmp.subset_rows(i);
@@ -601,8 +643,23 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 		out.srs = out.srs;
 		return out;
 	}
-	
-	
+
+	if (htype != "convex") {
+		#ifndef GEOS361
+		out.setError("GEOS 3.6.1 required for rotated rectangle");
+		return out;
+		#endif
+		if (is_lonlat()) {
+			if ((extent.ymin > -85) && (extent.ymax < 85)) {
+				SpatVector tmp = project("+proj=merc");
+				tmp = tmp.hull(htype, "");
+				tmp = tmp.project(srs.wkt);
+				return tmp;
+			}
+		}
+	}
+
+
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	SpatVector a = aggregate(false);
 	std::vector<GeomPtr> g = geos_geoms(&a, hGEOSCtxt);
@@ -611,7 +668,7 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 	if (htype == "convex") {
 		h = GEOSConvexHull_r(hGEOSCtxt, g[0].get());
 	} else {
-	#ifndef HAVE361
+	#ifndef GEOS361
 		out.setError("GEOS 3.6.1 required for rotated rectangle");
 		return out;
 	#else 
@@ -633,7 +690,7 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges) {
 	SpatVector out;
 
-#ifndef HAVE350
+#ifndef GEOS350
 	out.setError("GEOS 3.5 required for voronoi");
 	return out;
 #else 
@@ -693,7 +750,7 @@ SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges)
 SpatVector SpatVector::delauny(double tolerance, int onlyEdges) {
 	SpatVector out;
 
-#ifndef HAVE350
+#ifndef GEOS350
 	out.setError("GEOS 3.5 required for delauny");
 	return out;
 #else 
@@ -731,7 +788,7 @@ SpatGeom hullify(SpatVector b, bool ispoly) {
 		SpatVector g = b.subset_rows(range);
 		g = g.hull("convex");
 		part.addGeom(g.geoms[0]);
-	}	
+	}
 	part = part.aggregate(true);
 	return part.geoms[0];
 }
@@ -741,12 +798,15 @@ SpatVector lonlat_buf(SpatVector x, double dist, unsigned quadsegs, bool ispol,
 
 
 	if ((x.extent.ymin > -60) && (x.extent.ymax < 60) && ((x.extent.ymax - x.extent.ymin) < 1) && dist < 110000) {
+		SpatSRS insrs = x.srs;		
 		x.setSRS("+proj=merc");
 		double f = 0.5 - (dist / 220000);
 		double halfy = x.extent.ymin + f * (x.extent.ymax - x.extent.ymin);
 		std::vector<double> dd = destpoint_lonlat(0, halfy, 0, dist);
 		dist = dd[1] - halfy;
-		return x.buffer({dist}, quadsegs);
+		x = x.buffer({dist}, quadsegs);
+		x.srs = insrs;
+		return x;		
 	} 
 
 	SpatVector tmp;
@@ -757,19 +817,19 @@ SpatVector lonlat_buf(SpatVector x, double dist, unsigned quadsegs, bool ispol,
 		p = p.as_points(false, true);
 		std::vector<double> d(p.size(), dist);
 		SpatVector b = p.point_buffer(d, quadsegs, true);
-		if (b.size() <= p.size()) {	
+		if (b.size() <= p.size()) {
 			SpatGeom g = hullify(b, ispol);
 			tmp.addGeom(g);
-		} else {			
+		} else {
 			SpatVector west, east, eastwest;
 			for (size_t j =0; j<b.size(); j++) {
 				if ((b.geoms[j].extent.xmin < -179.99) && (b.geoms[j].extent.xmax > 179.99)) {
-					tmp.addGeom(b.geoms[j]);							
+					tmp.addGeom(b.geoms[j]);
 				} else if (b.geoms[j].extent.xmax < 0) {
-					west.addGeom(b.geoms[j]);					
+					west.addGeom(b.geoms[j]);
 				} else {
-					east.addGeom(b.geoms[j]);										
-				}	
+					east.addGeom(b.geoms[j]);
+				}
 			}
 			if (east.nrow() > 0) {
 				SpatGeom geast = hullify(east, ispol);
@@ -811,7 +871,7 @@ SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
 			if (dist[i] <= 0) {
 				dist[i] = -dist[i];
 			}
-		}	
+		}
 	}
 	recycle(dist, size());
 
@@ -839,14 +899,14 @@ SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
 					p = lonlat_buf(p, dist[i], quadsegs, false, false);
 				}
 				out = out.append(p, true);
-			}	
+			}
 			out.df = df;
-			return out;	
+			return out;
 		}
 	}
 
-	
-	
+
+
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 //	SpatVector f = remove_holes();
 
@@ -868,7 +928,7 @@ SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
 	out = coll.get(0);
 	out.srs = srs;
 	out.df = df;
-	
+
 	return out;
 }
 
@@ -908,12 +968,12 @@ SpatVector SpatVector::intersect(SpatVector v) {
 		//std::vector<int> sx;
 		//sx.reserve(count);
 		//for (size_t i=0; i<ixj.size(); i++) {
-		//	if (ixj[i]) sx.push_back(i);	
+		//	if (ixj[i]) sx.push_back(i);
 		//}
 		out = subset_rows(idx);
-		
+
 	} else {
-		
+
 		long k = 0;
 		for (size_t i = 0; i < nx; i++) {
 			for (size_t j = 0; j < ny; j++) {
@@ -1170,7 +1230,7 @@ std::vector<int> SpatVector::relate(std::string relation, bool symmetrical) {
 				for (size_t j=(i+1); j<s; j++) {
 					out.push_back( relFun(hGEOSCtxt, pr.get(), x[j].get()));
 				}
-			} 			
+			} 
 		}
 	} else {
 		size_t nx = size();
@@ -1226,7 +1286,7 @@ std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel) {
 			x = geos_geoms(&rr, hGEOSCtxt);
 			y = geos_geoms(&v, hGEOSCtxt);
 		}
-		
+
 	} else {
 		x = geos_geoms(this, hGEOSCtxt);
 		y = geos_geoms(&v, hGEOSCtxt);
@@ -1259,7 +1319,7 @@ std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel) {
 				if ( GEOSDistance_r(hGEOSCtxt, x[i].get(), y[0].get(), &d)) {
 					out.push_back(d);
 				} else {
-					out.push_back(NAN);				
+					out.push_back(NAN);
 				}
 			}
 		} else {
@@ -1268,7 +1328,7 @@ std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel) {
 				if ( GEOSDistance_r(hGEOSCtxt, x[i].get(), y[i].get(), &d)) {
 					out.push_back(d);
 				} else {
-					out.push_back(NAN);				
+					out.push_back(NAN);
 				}
 			}
 		}
@@ -1279,7 +1339,7 @@ std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel) {
 				if ( GEOSDistance_r(hGEOSCtxt, x[i].get(), y[j].get(), &d)) {
 					out.push_back(d);
 				} else {
-					out.push_back(NAN);				
+					out.push_back(NAN);
 				}
 			}
 		}
@@ -1303,7 +1363,7 @@ std::vector<double> SpatVector::geos_distance(bool sequential) {
 			if ( GEOSDistance_r(hGEOSCtxt, x[i].get(), x[i+1].get(), &d)) {
 				out.push_back(d);
 			} else {
-				out.push_back(NAN);				
+				out.push_back(NAN);
 			}
 		}
 	} else {
@@ -1313,14 +1373,14 @@ std::vector<double> SpatVector::geos_distance(bool sequential) {
 				if ( GEOSDistance_r(hGEOSCtxt, x[i].get(), x[j].get(), &d)) {
 					out.push_back(d);
 				} else {
-					out.push_back(NAN);				
+					out.push_back(NAN);
 				}
 			}
 		}
 	}
 	if (s == 1) {
 		out.push_back(0);
-	}	
+	}
 	geos_finish(hGEOSCtxt);
 	return out;
  }
@@ -1333,7 +1393,7 @@ SpatVector SpatVector::unite(SpatVector v) {
 	}
 	SpatVector sdif = symdif(v);
 	if (sdif.hasError()) {
-		return intsec;
+		return sdif;
 	}
 	return intsec.append(sdif, true);
 }
@@ -1354,6 +1414,9 @@ SpatVector SpatVector::unite() {
 		SpatVector r = subset_rows(i);
 		r.df = d;
 		out = out.unite(r);
+		if (out.hasError()) {
+			return out;
+		}		
 	}
 
 	for (size_t i=0; i<out.df.iv.size(); i++) {
@@ -1378,7 +1441,7 @@ SpatVector SpatVector::symdif(SpatVector v) {
 	SpatVector out = erase(v);
 	out = out.append(v.erase(*this), true);
 	return out;
-	
+
 /*
 	SpatVector out;
 	out.srs = srs;
@@ -1392,7 +1455,7 @@ SpatVector SpatVector::symdif(SpatVector v) {
 	size_t nx = size();
 	size_t ny = v.size();
 
-	
+
 	for (size_t i = 0; i < nx; i++) {
 		GEOSGeometry* geom = x[i].get();
 		for (size_t j = 0; j < ny; j++) {
@@ -1449,8 +1512,136 @@ SpatVector SpatVector::cover(SpatVector v, bool identity) {
 }
 
 
+SpatVector SpatVector::erase_agg(SpatVector v) {
+
+	if ((type() == "points") || (v.type() == "points")) {
+		std::vector<int> b = relateFirst(v, "intersects");
+		std::vector<unsigned> r;
+		r.reserve(b.size());
+		for (size_t i=0; i < b.size(); i++) {
+			if (b[i] == -1) r.push_back(i);	
+		}
+		return subset_rows(r);
+	}
+		
+	SpatVector out;
+
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
+
+// this approach is nicer than the below in ::erase 
+// but it fails if polys overlap
+	v = v.aggregate(false);
+	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
+	std::vector<unsigned> rids;
+	size_t nx = size();
+	std::vector<GeomPtr> result;
+
+	for (size_t i = 0; i < nx; i++) {
+		GEOSGeometry* geom = GEOSDifference_r(hGEOSCtxt, x[i].get(), y[0].get());
+		if (geom == NULL) {
+			out.setError("GEOS exception");
+			geos_finish(hGEOSCtxt);
+			return(out);
+		} 
+		if (GEOSisEmpty_r(hGEOSCtxt, geom)) {
+			GEOSGeom_destroy_r(hGEOSCtxt, geom);
+		} else {
+			result.push_back(geos_ptr(geom, hGEOSCtxt));
+			rids.push_back(i);
+		}
+	}
+	if (result.size() > 0) {
+		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
+		out = coll.get(0);
+		out.srs = srs;
+		out.df = df.subset_rows(rids);
+	} else {
+		out = subset_rows({-1});
+	}
+	geos_finish(hGEOSCtxt);
+	if (!srs.is_same(v.srs, true)) {
+		out.addWarning("different crs"); 
+	}
+	return out;
+}
+
+
+
 SpatVector SpatVector::erase(SpatVector v) {
 
+	if ((type() == "points") || (v.type() == "points")) {
+		std::vector<int> b = relateFirst(v, "intersects");
+		std::vector<unsigned> r;
+		r.reserve(b.size());
+		for (size_t i=0; i<b.size(); i++) {
+			if (b[i] == -1) r.push_back(i);	
+		}
+		return subset_rows(r);
+	}
+		
+	SpatVector out;
+
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
+	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
+	size_t nx = size();
+	size_t ny = v.size();
+	std::vector<int> rids;
+	rids.reserve(nx);
+	
+	for (size_t i = 0; i < nx; i++) {
+		bool good=true;
+		for (size_t j = 0; j < ny; j++) {
+			GEOSGeometry* geom = GEOSDifference_r(hGEOSCtxt, x[i].get(), y[j].get());
+			if (geom == NULL) {
+				out.setError("GEOS exception");
+				geos_finish(hGEOSCtxt);
+				return(out);
+			} 
+			if (GEOSisEmpty_r(hGEOSCtxt, geom)) {
+				GEOSGeom_destroy_r(hGEOSCtxt, geom);
+				good = false;
+				break;
+			}	
+			x[i] = geos_ptr(geom, hGEOSCtxt);
+		}
+		if (good) rids.push_back(i);
+	}
+
+	if (rids.size() > 0) {
+		SpatVectorCollection coll = coll_from_geos(x, hGEOSCtxt);
+		out = coll.get(0);
+		out.srs = srs;
+		out.df = df;
+		if (rids.size() != nx) {
+			out = out.subset_rows(rids);
+		}
+	} else {
+		out = subset_rows({-1});		
+	}
+	geos_finish(hGEOSCtxt);
+	if (!srs.is_same(v.srs, true)) {
+		out.addWarning("different crs"); 
+	}
+	return out;
+}
+
+
+
+/*
+SpatVector SpatVector::erase(SpatVector v) {
+
+	if ((type() == "points") || (v.type() == "points")) {
+		std::vector<int> b = relateFirst(v, "intersects");
+		std::vector<unsigned> r;
+		r.reserve(b.size());
+		for (size_t i=0; i<b.size(); i++) {
+			if (b[i] == -1) r.push_back(i);	
+		}
+		return subset_rows(r);
+	}
+		
 	SpatVector out;
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
@@ -1459,7 +1650,7 @@ SpatVector SpatVector::erase(SpatVector v) {
 	std::vector<unsigned> rids;
 	size_t nx = size();
 	size_t ny = v.size();
-	
+
 	for (size_t i = 0; i < nx; i++) {
 		//GEOSGeometry* geom = x[i].get();
 		for (size_t j = 0; j < ny; j++) {
@@ -1470,7 +1661,7 @@ SpatVector SpatVector::erase(SpatVector v) {
 				return(out);
 			} 
 			if (GEOSisEmpty_r(hGEOSCtxt, geom)) {
-				GEOSGeom_destroy_r(hGEOSCtxt, geom);	
+				GEOSGeom_destroy_r(hGEOSCtxt, geom);
 				rids.push_back(i);
 				break;
 			}
@@ -1494,11 +1685,11 @@ SpatVector SpatVector::erase(SpatVector v) {
 	return out;
 }
 
-
+*/
 
 SpatVector SpatVector::erase() {
 	SpatVector out;
-	
+
 	if (type() != "polygons") {
 		out.setError("not polygons");
 		return out;
@@ -1512,7 +1703,7 @@ SpatVector SpatVector::erase() {
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 	std::vector<unsigned> rids;
 
-	for (size_t i = 0; i < (n-1); i++) {		
+	for (size_t i = 0; i < (n-1); i++) {
 		for (size_t j = (i+1); j < n; j++) {
 			GEOSGeometry* geom = GEOSDifference_r(hGEOSCtxt, x[i].get(), x[j].get());
 			if (geom == NULL) {
@@ -1520,13 +1711,13 @@ SpatVector SpatVector::erase() {
 				geos_finish(hGEOSCtxt);
 				return(out);
 			} else if (GEOSisEmpty_r(hGEOSCtxt, geom)) {
-				GEOSGeom_destroy_r(hGEOSCtxt, geom);	
+				GEOSGeom_destroy_r(hGEOSCtxt, geom);
 				rids.push_back(i);
 				break;
 			} else {
 				x[i] = geos_ptr(geom, hGEOSCtxt);
 			}
-		}	
+		}
 	}
 
 	SpatVectorCollection coll = coll_from_geos(x, hGEOSCtxt);
@@ -1545,24 +1736,27 @@ SpatVector SpatVector::erase() {
 
 SpatVector SpatVector::gaps() {
 	SpatVector out;
-	
+
 	if (type() != "polygons") {
 		out.setError("not polygons");
 		return out;
 	}
+	
 	size_t n = size();
 	if (n < 2) {
 		out.srs = srs;
 		return out;
 	}
-
+	out = aggregate(true);
+	return out.get_holes();
+/*
 	SpatExtent e = extent;
 	e.xmin -= 11;
 	e.xmax += 10;
 	e.ymin -= 10;
 	e.ymax += 10;
 	SpatVector p(e, "");
-	
+
 	p = p.erase(*this);
 	p = p.disaggregate();
 	double exmin = e.xmin + 1;
@@ -1576,12 +1770,13 @@ SpatVector SpatVector::gaps() {
 	std::vector<unsigned> r(1, j);
 	p.srs = srs;
 	return p.remove_rows(r);
+*/
 }
 
 
 SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 	SpatVector out;
-		
+
 	if ((size() == 0) || (v.size()==0)) {
 		out.setError("empty SpatVecor(s)");
 		return out;
@@ -1591,7 +1786,7 @@ SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 		return out;
 	}
 	out.srs = srs;
-	
+
 	if (is_lonlat()) {
 		if (type() == "points") {
 			std::vector<double> nlon, nlat, dist;
@@ -1604,16 +1799,16 @@ SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 			std::iota(fromid.begin(), fromid.end(), 0);
 			out.df.add_column(fromid, "from_id");
 			out.df.add_column(p[0], "from_x");
-			out.df.add_column(p[1], "from_y");	
+			out.df.add_column(p[1], "from_y");
 			out.df.add_column(id, "to_id");
 			out.df.add_column(nlon, "to_x");
 			out.df.add_column(nlat, "to_y");
-			out.df.add_column(dist, "distance");					
+			out.df.add_column(dist, "distance");
 			return out;
 		} else {
 			out.setError("not yet implement for non-point lonlat vector data");
 			return out;
-		}		
+		}
 	}
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
@@ -1621,23 +1816,23 @@ SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 		if ((size() != v.size())) {
 			out.setError("SpatVecors do not have the same size");
 			return out;
-		}		
+		}
 		std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 		std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
 		std::vector<GeomPtr> b(size());
-		for (size_t i=0; i < x.size(); i++) {	
+		for (size_t i=0; i < x.size(); i++) {
 			GEOSCoordSequence* csq = GEOSNearestPoints_r(hGEOSCtxt, x[i].get(), y[i].get());
 			GEOSGeometry* geom = GEOSGeom_createLineString_r(hGEOSCtxt, csq);
 			b[i] = geos_ptr(geom, hGEOSCtxt);
 		}
 		out = vect_from_geos(b, hGEOSCtxt, "lines");
-		
-	} else {	
+
+	} else {
 		SpatVector mp = v.aggregate(false);
 		std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 		std::vector<GeomPtr> y = geos_geoms(&mp, hGEOSCtxt);
 		std::vector<GeomPtr> b(size());
-		for (size_t i = 0; i < x.size(); i++) {	
+		for (size_t i = 0; i < x.size(); i++) {
 			GEOSCoordSequence* csq = GEOSNearestPoints_r(hGEOSCtxt, x[i].get(), y[0].get());
 			GEOSGeometry* geom = GEOSGeom_createLineString_r(hGEOSCtxt, csq);
 			b[i] = geos_ptr(geom, hGEOSCtxt);
@@ -1661,7 +1856,7 @@ SpatVector SpatVector::nearest_point() {
 	}
 	size_t n = size();
 	out.srs = srs;
-	
+
 	if (is_lonlat()) {
 		if (type() == "points") {
 			std::vector<double> nlon, nlat, dist;
@@ -1670,19 +1865,19 @@ SpatVector SpatVector::nearest_point() {
 			nearest_lonlat_self(id, dist, nlon, nlat, p[0], p[1]);
 			out.setPointsGeometry(nlon, nlat);
 			out.df.add_column(id, "id");
-			out.df.add_column(dist, "distance");		
+			out.df.add_column(dist, "distance");
 			return out;
 		} else {
 			out.setError("not yet implement for non-point lonlat vector data");
 			return out;
-		}		
+		}
 	}
 
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> b(n);
-	for (unsigned i = 0; i < n; i++) {	
+	for (unsigned i = 0; i < n; i++) {
 		SpatVector xa = remove_rows({i});
 		xa = xa.aggregate(false);
 		std::vector<GeomPtr> y = geos_geoms(&xa, hGEOSCtxt);
@@ -1696,16 +1891,64 @@ SpatVector SpatVector::nearest_point() {
 	return out;
 }
 
+SpatVector SpatVector::cross_dateline(bool &fixed) {
+	SpatVector out;
+	fixed = false;
+	if (type() == "points") {
+		return out;
+	}
+
+	for (size_t i=0; i<geoms.size(); i++) {
+		if ((geoms[i].size() > 1) && 
+			((geoms[i].extent.xmax - geoms[i].extent.xmin) > 180)) {
+			SpatGeom g = geoms[i];
+			for (size_t j=0; j<g.size(); j++) {
+				if (g.parts[j].extent.xmax < 0) {
+					for (size_t k=0; k<g.parts[j].x.size(); k++) {
+						g.parts[j].x[k] += 360;
+					}
+					for (size_t k=0; k<g.parts[j].holes.size(); k++) {
+						for (size_t m=0; m<g.parts[j].holes[k].x.size(); m++) {
+							g.parts[j].holes[k].x[m] += 360;
+						}
+					}
+					g.parts[j].extent.xmin += 360;
+					g.parts[j].extent.xmax += 360;
+					g.setPart(g.parts[j], j);
+					fixed = true;
+				}
+			}
+			out.addGeom(g);
+		} else {
+			out.addGeom(geoms[i]);
+		}
+	}
+	out.srs = srs;
+	out.df = df;
+	return out;
+}
 
 
-SpatVector SpatVector::centroid() {
+
+
+SpatVector SpatVector::centroid(bool check_lonlat) {
 
 	SpatVector out;
 
+	if (check_lonlat && could_be_lonlat()) {
+		bool changed = false;
+		SpatVector v = cross_dateline(changed);
+		if (changed) {
+			out = v.centroid(false);
+			out.fix_lonlat_overflow();
+			return out;
+		}
+	}
+
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> b(size());
-	for (size_t i = 0; i < g.size(); i++) {	
+	for (size_t i = 0; i < g.size(); i++) {
 		GEOSGeometry* pt = GEOSGetCentroid_r(hGEOSCtxt, g[i].get());
 		if (pt == NULL) {
 			out.setError("NULL geom");
@@ -1722,13 +1965,14 @@ SpatVector SpatVector::centroid() {
 	return out;
 }
 
+
 SpatVector SpatVector::unaryunion() {
 	SpatVector out;
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> gout(g.size());
-	for (size_t i = 0; i < g.size(); i++) {	
+	for (size_t i = 0; i < g.size(); i++) {
 		GEOSGeometry* u = GEOSUnaryUnion_r(hGEOSCtxt, g[i].get());
 		if (u == NULL) {
 			out.setError("NULL geom");
@@ -1762,18 +2006,18 @@ bool geos_buffer(GEOSContextHandle_t hGEOSCtxt, std::vector<GeomPtr> &g, double
 
 
 SpatVector SpatVector::width() {
-	
+
 	SpatVector tmp;
-	
-#ifndef HAVE361
+
+#ifndef GEOS361
 	tmp.setError("GEOS 3.6.1 required for width");
 	return tmp;
 #else 
-	
+
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> gout(g.size());
-	for (size_t i = 0; i < g.size(); i++) {	
+	for (size_t i = 0; i < g.size(); i++) {
 		GEOSGeometry* w = GEOSMinimumWidth_r(hGEOSCtxt, g[i].get());
 		if (w == NULL) {
 			tmp.setError("found NULL geom");
@@ -1787,23 +2031,21 @@ SpatVector SpatVector::width() {
 	tmp = coll.get(0);
 	tmp.srs = srs;
 	return tmp;
-	
+
 #endif
 }
 
 SpatVector SpatVector::clearance() {
-
 	SpatVector tmp;
-
-#ifndef HAVE361
-	tmp.setError("GEOS 3.5 required for clearance");
+#ifndef GEOS361
+	tmp.setError("GEOS 3.6 required for clearance");
 	return tmp;
 #else 
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> gout(g.size());
-	for (size_t i = 0; i < g.size(); i++) {	
+	for (size_t i = 0; i < g.size(); i++) {
 		GEOSGeometry* w = GEOSMinimumClearanceLine_r(hGEOSCtxt, g[i].get());
 		if (w == NULL) {
 			tmp.setError("NULL geom");
@@ -1821,4 +2063,3 @@ SpatVector SpatVector::clearance() {
 #endif
 }
 
-
