@@ -1,3 +1,4 @@
+
 #include "ogr_spatialref.h"
 
 #include "spatRaster.h"
@@ -31,8 +32,9 @@ SpatRaster SpatRaster::hardCopy(SpatOptions &opt) {
 		return out;
 	}
 	for (size_t i = 0; i < out.bs.n; i++) {
-		std::vector<double> v = readBlock(out.bs, i);
-		if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i], 0, ncol())) return out;
+		std::vector<double> v; 
+		readBlock(v, out.bs, i);
+		if (!out.writeBlock(v, i)) return out;
 	}
 	out.writeStop();
 	readStop();
@@ -62,7 +64,7 @@ bool SpatRaster::getDSh(GDALDatasetH &rstDS, SpatRaster &out, std::string &filen
 		if (canProcessInMemory(opt)) {
 			driver = "MEM";
 		} else {
-			filename = tempFile(opt.get_tempdir(), ".tif");
+			filename = tempFile(opt.get_tempdir(), opt.pid, ".tif");
 			opt.set_filenames({filename});
 			driver = "GTiff";
 		} 
@@ -77,9 +79,14 @@ bool SpatRaster::getDSh(GDALDatasetH &rstDS, SpatRaster &out, std::string &filen
 		if (!can_write(filename, opt.get_overwrite(), msg)) {
 			out.setError(msg);
 			return false;
-		}	
+		}
 	}
 
+	if (opt.names.size() == nlyr()) {
+		out.setNames(opt.names);
+	}
+
+
 	if (update) {
 		out = hardCopy(opt);
 		//size_t ns = source.size();
@@ -111,7 +118,7 @@ SpatRaster SpatRaster::rasterizeLyr(SpatVector x, double value, double backgroun
 	out.setNames({"ID"});
 
 	if ( !hasValues() ) update = false;
-	if (update) { // all lyrs		
+	if (update) { // all lyrs
 		out = geometry();
 	} else {
 		out = geometry(1);
@@ -142,11 +149,11 @@ SpatRaster SpatRaster::rasterizeLyr(SpatVector x, double value, double backgroun
 		// passing NULL instead may also work.
 		value = naval;
 	}
-	
+
 	std::vector<int> bands(out.nlyr());
 	std::iota(bands.begin(), bands.end(), 1);
 	std::vector<double> values(out.nlyr(), value);
-	
+
 	char** papszOptions = NULL;
 	CPLErr err;
 	if (touches) {
@@ -154,26 +161,26 @@ SpatRaster SpatRaster::rasterizeLyr(SpatVector x, double value, double backgroun
 	}
 	err = GDALRasterizeLayers(rstDS, static_cast<int>(bands.size()), &(bands[0]),
 			1, &(ahLayers[0]), NULL, NULL, &(values[0]), papszOptions, NULL, NULL);
-			
-	CSLDestroy(papszOptions);	
-			
+
+	CSLDestroy(papszOptions);
+
 //	for (size_t i=0; i<ahGeometries.size(); i++) {
-//		OGR_G_DestroyGeometry(ahGeometries[i]);			
+//		OGR_G_DestroyGeometry(ahGeometries[i]);
 //	}
 	GDALClose(vecDS);
-	
+
 	if ( err != CE_None ) {
 		out.setError("rasterization failed");
 		GDALClose(rstDS);
 		return out;
 	}
-	
+
 	if (driver == "MEM") {
 		if (!out.from_gdalMEM(rstDS, false, true)) {
 			out.setError("rasterization failed (mem)");
 		}
 	}
-	
+
 	GDALRasterBandH band = GDALGetRasterBand(rstDS, 1);
 	double adfMinMax[2];
 	GDALComputeRasterMinMax(band, false, adfMinMax);
@@ -193,7 +200,7 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 	std::string gtype = x.type();
 	bool ispol = gtype == "polygons";
 	if (weights) update = false;
-	
+
 	if (weights && ispol) {
 		SpatOptions sopts(opt);
 		SpatRaster wout = geometry(1);
@@ -215,7 +222,11 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		out = hardCopy(opt);
 	} else {
 		out = geometry(1);
-		out.setNames({field});
+		if (field == "") {
+			out.setNames({"layer"});
+		} else {
+			out.setNames({field});
+		}
 	}
 
 	if (ispol && touches && add) {
@@ -228,7 +239,7 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		if (i < 0) {
 			out.setError("field " + field + " not found");
 			return out;
-		}		
+		}
 		std::string dt = x.df.get_datatype(field);
 		if (dt == "string") {
 			//std::vector<std::string> ss = ;
@@ -252,7 +263,7 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 			values.resize(v.size());
 			for (size_t i=0; i<values.size(); i++) {
 				values[i] = v[i];
-			}			
+			}
 		}
 	}
 	size_t nGeoms = x.size();
@@ -265,24 +276,21 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		out.setError(x.getError());
 		return out;
 	}
-    std::vector<OGRGeometryH> ahGeometries;
+    std::vector<OGRGeometry *> ogrGeoms;
+	ogrGeoms.reserve(nGeoms);
+	
 	OGRLayer *poLayer = vecDS->GetLayer(0);
 	poLayer->ResetReading();
-	OGRFeature *poFeature;
 	
-
+	OGRFeature *poFeature;
 	while( (poFeature = poLayer->GetNextFeature()) != NULL ) {
-		OGRGeometry *poGeometry = poFeature->StealGeometry();
-#if GDAL_VERSION_MAJOR <= 2 && GDAL_VERSION_MINOR <= 2
-        OGRGeometryH hGeom = poGeometry;
-#else
-		OGRGeometryH hGeom = poGeometry->ToHandle(poGeometry);
-#endif
-		ahGeometries.push_back( hGeom );
+		OGRGeometry *poGeometry = poFeature->GetGeometryRef();
+		if (poGeometry != NULL) {
+			OGRGeometry *copyGeom = poGeometry->clone();
+			ogrGeoms.push_back( copyGeom );
+		}
+		OGRFeature::DestroyFeature( poFeature );
 	}
-//	OGRGeometryFactory::destroyGeometry(poGeometry); ?
-
-	OGRFeature::DestroyFeature( poFeature );
 	GDALClose(vecDS);
 
 	std::string errmsg, driver, filename;
@@ -296,12 +304,10 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 	for (double &d : values) d = std::isnan(d) ? naval : d;
 		// passing NULL instead may also work.
 
-
 	std::vector<int> bands(out.nlyr());
 	std::iota(bands.begin(), bands.end(), 1);
 	rep_each(values, out.nlyr());
 
-
 	char** papszOptions = NULL;
 	CPLErr err;
 	if (ispol && touches && (nGeoms > 1)) {
@@ -309,15 +315,16 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		papszOptions = CSLSetNameValue(papszOptions, "ALL_TOUCHED", "TRUE"); 
 		err = GDALRasterizeGeometries(rstDS, 
 				static_cast<int>(bands.size()), &(bands[0]),
-				static_cast<int>(ahGeometries.size()), &(ahGeometries[0]),
-				NULL, NULL, &(values[0]), papszOptions, NULL, NULL);		
-		CSLDestroy(papszOptions);	
+				static_cast<int>(ogrGeoms.size()), 
+				(OGRGeometryH *) &(ogrGeoms[0]),
+				NULL, NULL, &(values[0]), papszOptions, NULL, NULL);
+		CSLDestroy(papszOptions);
 
 		if ( err != CE_None ) {
 			out.setError("rasterization failed");
 			GDALClose(rstDS);
-			for (size_t i=0; i<ahGeometries.size(); i++) {
-				OGR_G_DestroyGeometry(ahGeometries[i]);			
+			for (size_t i=0; i<ogrGeoms.size(); i++) {
+				OGR_G_DestroyGeometry(ogrGeoms[i]);
 			}
 			return out;
 		}
@@ -325,7 +332,8 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		// second time to fix the internal area
 		err = GDALRasterizeGeometries(rstDS, 
 				static_cast<int>(bands.size()), &(bands[0]),
-				static_cast<int>(ahGeometries.size()), &(ahGeometries[0]),
+				static_cast<int>(ogrGeoms.size()), 
+				(OGRGeometryH *) &(ogrGeoms[0]),
 				NULL, NULL, &(values[0]), NULL, NULL, NULL);
 
 	} else {
@@ -336,36 +344,37 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		}
 		err = GDALRasterizeGeometries(rstDS, 
 				static_cast<int>(bands.size()), &(bands[0]),
-				static_cast<int>(ahGeometries.size()), &(ahGeometries[0]),
+				static_cast<int>(ogrGeoms.size()), 
+				(OGRGeometryH *) &(ogrGeoms[0]),
 				NULL, NULL, &(values[0]), papszOptions, NULL, NULL);
-				
-		CSLDestroy(papszOptions);	
+
+		CSLDestroy(papszOptions);
 	}
-			
-	for (size_t i=0; i<ahGeometries.size(); i++) {
-		OGR_G_DestroyGeometry(ahGeometries[i]);			
+
+	for (size_t i=0; i<ogrGeoms.size(); i++) {
+		OGR_G_DestroyGeometry(ogrGeoms[i]);
 	}
-	
+
 	if ( err != CE_None ) {
 		out.setError("rasterization failed");
 		GDALClose(rstDS);
 		return out;
 	}
-	
+
 	if (driver == "MEM") {
 		if (!out.from_gdalMEM(rstDS, false, true)) {
 			out.setError("rasterization failed (mem)");
 		}
 	}
-	
+
 	GDALRasterBandH band = GDALGetRasterBand(rstDS, 1);
-	
+
 	if (minmax) {
 		double adfMinMax[2];
 		GDALComputeRasterMinMax(band, false, adfMinMax);
 		GDALSetRasterStatistics(band, adfMinMax[0], adfMinMax[1], -9999, -9999);
 	}
-	
+
 	GDALClose(rstDS);
 	if (driver != "MEM") {
 		out = SpatRaster(filename, {-1}, {""}, {});
@@ -381,7 +390,7 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 
 std::vector<double> SpatRaster::rasterizeCells(SpatVector &v, bool touches, SpatOptions &opt) { 
 // note that this is only for lines and polygons
-    SpatOptions ropt;
+    SpatOptions ropt(opt);
 	SpatRaster r = geometry(1);
 	SpatExtent e = getExtent();
 	e.intersect(v.getExtent());
@@ -389,9 +398,9 @@ std::vector<double> SpatRaster::rasterizeCells(SpatVector &v, bool touches, Spat
 		std::vector<double> out(1, NAN);
 		return out;
 	}
-	
+
 	SpatRaster rc = r.crop(e, "out", ropt);
-	std::vector<double> feats(1, 1) ;		
+	std::vector<double> feats(1, 1) ;
     SpatRaster rcr = rc.rasterize(v, "", feats, NAN, touches, false, false, false, false, ropt); 
 	SpatVector pts = rcr.as_points(false, true, ropt);
 	std::vector<double> cells;
@@ -405,7 +414,7 @@ std::vector<double> SpatRaster::rasterizeCells(SpatVector &v, bool touches, Spat
 		if (cells.size() == 0) {
 			cells.resize(1, NAN);
 		}
-	} else {	
+	} else {
 		SpatDataFrame vd = pts.getGeometryDF();
 		std::vector<double> x = vd.getD(0);
 		std::vector<double> y = vd.getD(1);
@@ -428,7 +437,7 @@ void SpatRaster::rasterizeCellsWeights(std::vector<double> &cells, std::vector<d
 	}
 	SpatRaster r = rr.crop(v.extent, "out", ropt);
 	r = r.disaggregate(fact, ropt);
-	std::vector<double> feats(1, 1) ;	
+	std::vector<double> feats(1, 1) ;
 	r = r.rasterize(v, "", feats, NAN, true, false, false, false, false, ropt); 
 	r = r.arith(100.0, "/", false, ropt);
 	r = r.aggregate(fact, "sum", true, ropt);
@@ -456,13 +465,13 @@ void SpatRaster::rasterizeCellsExact(std::vector<double> &cells, std::vector<dou
 	r = r.crop(v.extent, "out", ropt);
 
 //	if (r.ncell() < 1000) {
-		std::vector<double> feats(1, 1) ;	
+		std::vector<double> feats(1, 1) ;
 		r = r.rasterize(v, "", feats, NAN, true, false, false, false, false, ropt); 
 
 		SpatVector pts = r.as_points(true, true, ropt);
 		if (pts.size() == 0) {
 			weights.resize(1);
-			weights[0] = NAN;			
+			weights[0] = NAN;
 			cells.resize(1);
 			cells[0] = NAN;
 		} else {
@@ -489,7 +498,7 @@ void SpatRaster::rasterizeCellsExact(std::vector<double> &cells, std::vector<dou
 // because touches=false includes partly overlapped cells [#346]
 
 	else {
-		std::vector<double> feats(1, 1) ;	
+		std::vector<double> feats(1, 1) ;
 		SpatVector vv = v.as_lines();
 		SpatRaster b = r.rasterize(vv, "", feats, NAN, true, false, false, false, false, opt); 
 		SpatVector pts = b.as_points(true, true, opt);
@@ -498,7 +507,7 @@ void SpatRaster::rasterizeCellsExact(std::vector<double> &cells, std::vector<dou
 			std::vector<double> x = vd.getD(0);
 			std::vector<double> y = vd.getD(1);
 			cells = cellFromXY(x, y);
-			
+
 			SpatVector bv = b.as_polygons(false, false, false, true, opt);
 			std::vector<double> csize = bv.area("m", true, {});
 			bv.df.add_column(csize, "cellsize");
@@ -519,18 +528,19 @@ void SpatRaster::rasterizeCellsExact(std::vector<double> &cells, std::vector<dou
 			std::vector<double> y = vd.getD(1);
 			std::vector<double> cells2 = cellFromXY(x, y);
 			cells.insert(cells.end(), cells2.begin(), cells2.end());
-			weights.resize(weights.size() + cells2.size(), 1);	
+			weights.resize(weights.size() + cells2.size(), 1);
 		}
-		
+
 		if (cells.size() == 0) {
 			weights.resize(1);
-			weights[0] = NAN;			
+			weights[0] = NAN;
 			cells.resize(1);
 			cells[0] = NAN;
 		}
 	}
-*/	
-	
+*/
+
 }
 
 
+
