@@ -19,11 +19,12 @@
 #include <algorithm>
 #include <stdint.h>
 #include <vector>
-#include <regex>
+//#include <regex>
 
 //#include "spatRaster.h"
 #include "spatRasterMultiple.h"
 
+#include "vecmath.h"
 #include "file_utils.h"
 #include "string_utils.h"
 #include "spatTime.h"
@@ -57,6 +58,87 @@ void SpatRaster::gdalogrproj_init(std::string path) {
 #endif
 }
 
+/*
+bool GetTime(std::string filename, std::vector<int_64> &time, std::string &timestep, size_t nl) {
+	filename += ".time";
+	if (!file_exists(filename)) {
+		return false;
+	}
+	std::vector<std::string> s = read_text(filename);
+	if (nl != (s.size()-1)) return false;
+	time.reserve(nl);
+	timestep = s[0];
+	for (size_t i=0; i<nl; i++) {
+		time.push_back( parse_time(s[i+1]) );
+	}
+	return true;
+}
+
+bool GetUnits(std::string filename, std::vector<std::string> &units, size_t nl) {
+	filename += ".unit";
+	if (!file_exists(filename)) {
+		return false;
+	}
+	units = read_text(filename);
+	if (nl != units.size()) return false;
+	return true;
+}
+*/
+
+bool read_aux_json(std::string filename, std::vector<int_64> &time, std::string &timestep, std::vector<std::string> &units, size_t nlyr) {
+	filename += ".aux.json";
+	if (!file_exists(filename)) return false;
+	std::vector<std::string> s = read_text(filename);
+	int itime=-1, istep=-1, iunit=-1;
+	for (size_t i=0; i<s.size(); i++) {
+		std::vector<std::string> x = strsplit_first(s[i], ":");
+		if (x.size() != 2) continue;
+		x[0].erase(std::remove(x[0].begin(), x[0].end(), '\"'), x[0].end());
+		if (x[0] == "time") itime = i;
+		if (x[0] == "timestep") istep = i;
+		if (x[0] == "unit") iunit = i;
+	}
+	if (itime >= 0) {
+		std::vector<std::string> x = strsplit_first(s[itime], "[");
+		if (x.size() == 2) {
+			x = strsplit(x[1], "]");
+			x = strsplit(x[0], ",");
+			std::vector<int_64> tm;
+			for (size_t i=0; i<x.size(); i++) {
+				unquote(x[i]); 
+				tm.push_back( parse_time(x[i]) );
+			}
+			if (tm.size() == nlyr) {
+				time = tm;
+			}
+		}
+		if ((istep >= 0) && (time.size() > 0)) {
+			std::vector<std::string> x = strsplit_first(s[istep], ":");
+			if (x.size() == 2) {
+				x = strsplit(x[1], ",");
+				unquote(x[0]); 
+				timestep = x[0];
+			}
+		}
+	}
+	if (iunit >= 0) {
+		std::vector<std::string> x = strsplit_first(s[iunit], "[");
+		if (x.size() == 2) {
+			x = strsplit(x[1], "]");
+			x = strsplit(x[0], ",");
+			if (x.size() == nlyr) {
+				for (size_t i=0; i< x.size(); i++) {
+					unquote(x[i]); 
+				}
+				units = x;
+			}
+		}
+	}
+	return false;
+}
+
+
+
 SpatCategories GetRAT(GDALRasterAttributeTable *pRAT) {
 
 	SpatCategories out;
@@ -72,20 +154,45 @@ SpatCategories GetRAT(GDALRasterAttributeTable *pRAT) {
 	size_t nc = (int) pRAT->GetColumnCount();
 	size_t nr = (int) pRAT->GetRowCount();
 
-	std::vector<long> id(nr);
-	std::iota(id.begin(), id.end(), 0);
-	out.d.add_column(id, "ID");
 
-	std::vector<std::string> ss = {"histogram", "red", "green", "blue", "opacity"};
+	std::vector<std::string> ss = {"histogram", "count", "red", "green", "blue", "opacity", "r", "g", "b", "alpha"};
+
+	std::vector<std::string> ratnms;
+	std::vector<int> id, id2;
 	
+	bool hasvalue=false;
 	for (size_t i=0; i<nc; i++) {
+		std::string name = pRAT->GetNameOfCol(i);
+		lowercase(name);
+		ratnms.push_back(name);
+		if (name == "value") {
+			id.insert(id.begin(), i);
+			hasvalue = true;
+		} else {
+			int k = where_in_vector(name, ss, true);
+			if (k >= 0) {
+				id2.push_back(i);
+			} else {
+				id.push_back(i);	
+			}			
+		}
+	}
+	id.insert(id.end(), id2.begin(), id2.end());
+
+	if (!hasvalue) {
+		std::vector<long> vid(nr);
+		std::iota(vid.begin(), vid.end(), 0);
+		out.d.add_column(vid, "value");
+	}
+	
+	for (size_t k=0; k<id.size(); k++) {
+		size_t i = id[k];
+		std::string name = pRAT->GetNameOfCol(i);
+			
 		GDALRATFieldType nc_type = pRAT->GetTypeOfCol(i);
 //		GFT_type.push_back(GFU_type_string[nc_types[i]]);
 //		GDALRATFieldUsage nc_usage = pRAT->GetUsageOfCol(i);
 //		GFT_usage.push_back(GFU_usage_string[nc_usages[i]]);
-		std::string name = pRAT->GetNameOfCol(i);
-		int j = where_in_vector(name, ss, true);
-		if (j >= 0) continue;
 
 		if (nc_type == GFT_Integer) {
 			std::vector<long> d(nr);
@@ -107,34 +214,39 @@ SpatCategories GetRAT(GDALRasterAttributeTable *pRAT) {
 			out.d.add_column(d, name);
 		}
 	}
-	out.index = out.d.ncol() > 1 ? 1 : 0;
+	out.index = out.d.ncol() > 1 ? 1 : 0;	
 	return(out);
 }
 
 
 bool GetVAT(std::string filename, SpatCategories &vat) {
 
-	filename = filename + ".vat.dbf";
+	filename += ".vat.dbf";
 	if (!file_exists(filename)) {
 		return false;
 	}
 
 	SpatVector v, fvct;
 	std::vector<double> fext;
-	
+
 	v.read(filename, "", "", fext, fvct);
 	if (v.df.nrow() == 0) return false;
-	
-	std::vector<std::string> ss = {"histogram", "red", "green", "blue", "opacity"};
+
+
 	std::vector<std::string> nms = v.df.get_names();
+	std::vector<std::string> ss = {"count", "histogram"};
+
 	std::vector<unsigned> rng;
+	rng.reserve(nms.size());
+
 	for (size_t i=0; i<nms.size(); i++) {
 		int j = where_in_vector(nms[i], ss, true);
 		if (j < 0) rng.push_back(i);
 	}
+
 	if (rng.size() > 1) {
 		vat.d = v.df.subset_cols(rng);
-		vat.d.names[0] = "ID";
+//		vat.d.names[0] = "ID";
 		vat.index = 1;
 		std::string sc = vat.d.names[1];
 		lowercase(sc);
@@ -151,30 +263,138 @@ bool GetVAT(std::string filename, SpatCategories &vat) {
 	return false;
 }
 
+
 SpatDataFrame GetCOLdf(GDALColorTable *pCT) {
 
 	SpatDataFrame out;
 	size_t nc = (int) pCT->GetColorEntryCount();
 
+	out.add_column(1, "value");
 	out.add_column(1, "red");
 	out.add_column(1, "green");
 	out.add_column(1, "blue");
 	out.add_column(1, "alpha");
 	out.reserve(nc);
 
-	for (size_t i=0; i<nc; i++) {	
+	for (size_t i=0; i<nc; i++) {
 		const GDALColorEntry * col = pCT->GetColorEntry(i);
-		out.iv[0].push_back(col->c1);
-		out.iv[1].push_back(col->c2);
-		out.iv[2].push_back(col->c3);
-		out.iv[3].push_back(col->c4);
+		out.iv[0].push_back(i);
+		out.iv[1].push_back(col->c1);
+		out.iv[2].push_back(col->c2);
+		out.iv[3].push_back(col->c3);
+		out.iv[4].push_back(col->c4);
 	}
 	return(out);
 }
 
+bool getIntFromDoubleCol(std::vector<double> & dv, std::vector<long> &iv) {
+	double dmn = vmin(dv, true);
+	if (dmn < 0) return false;
+	double dmx = vmax(dv, true);
+	if (dmx > 255) {
+		return false;
+	}	
+	iv.resize(0);
+	iv.reserve(dv.size());
+	if (dmx <= 1) {
+		for (size_t i=0; i<dv.size(); i++) {
+			iv.push_back( dv[i] * 255 );
+		}
+	} else {
+		for (size_t i=0; i<dv.size(); i++) {
+			iv.push_back( dv[i] );
+		}			
+	}
+	return true;
+}
+
+bool setIntCol(SpatDataFrame &d, SpatDataFrame &out, int k, std::string name) {
+	if (d.itype[k] == 0) {
+		std::vector<long> iv;
+		size_t j = d.iplace[k];
+		if (getIntFromDoubleCol(d.dv[j], iv)) {
+			out.add_column(iv, name);
+		} else {
+			return false;
+		}	
+	} else if (d.itype[k] == 1) {
+		size_t j = d.iplace[k];
+		long dmn = vmin(d.iv[j], true);
+		if (dmn < 0) return false;
+		long dmx = vmax(d.iv[j], true);
+		if (dmx > 255) return false;
+		out.add_column(d.iv[j], name);			
+	} else {
+		return false;
+	}
+	return true;
+}
+	
+
+bool colsFromRat(SpatDataFrame &d, SpatDataFrame &out) {
+
+	std::vector<std::string> ss = d.get_names();
+	for (size_t i=0; i<ss.size(); i++) {
+		lowercase(ss[i]);
+	}
+
+	int k = where_in_vector("value", ss, true);
+	if (k >= 0) {
+		size_t j = d.iplace[k];
+		if (d.itype[k] == 1) {
+			out.add_column(d.iv[j], "value");
+		} else {
+			std::vector<long> x;
+			x.reserve(d.nrow());
+			for (size_t i=0; i<d.nrow(); i++) {
+				x.push_back(d.dv[j][i]);
+			}
+			out.add_column(x, "value");			
+		}
+	} else {
+		return false;
+	}
+
+	std::vector<std::string> cols1 = {"red", "green", "blue"};
+	std::vector<std::string> cols2 = {"r", "g", "b"};
+	for (size_t i=0; i<3; i++) {
+		int k = where_in_vector(cols1[i], ss, true);
+		if (k >= 0) {
+			if (!setIntCol(d, out, k, cols1[i])) return false;
+		} else {
+			int k = where_in_vector(cols2[i], ss, true);
+			if (k >= 0) {
+				if (!setIntCol(d, out, k, cols1[i])) return false;
+			} else {
+				return false;
+			}
+		}
+	}
+	
+	bool have_alpha = false;
+	k = where_in_vector("alpha", ss, true);
+	if (k >= 0) {
+		if (setIntCol(d, out, k, "alpha")) have_alpha = true;
+	}
+	if (!have_alpha) {
+		int k = where_in_vector("transparency", ss, true);
+		if (k >= 0) {
+			if (setIntCol(d, out, k, "alpha")) have_alpha = true;
+		}
+	}
+	if (!have_alpha) {
+		std::vector<long> a(out.nrow(), 255);
+		out.add_column(a, "alpha");
+	}
+
+	d = out;
+	return true;
+}
+
+
 /*
 SpatDataFrame GetColFromRAT(SpatDataFrame &rat) {
-	
+
 	SpatDataFrame out;
 	size_t nr = rat.nrow();
 	if (nr > 256) return out;
@@ -192,7 +412,7 @@ SpatDataFrame GetColFromRAT(SpatDataFrame &rat) {
 		std::vector<long> a(nr, 255);
 		out.add_column(a, "alpha");
 	}
-	out.names = {"red", "green", "blue", "alpha"};		
+	out.names = {"red", "green", "blue", "alpha"};
 	return out;
 }
 */
@@ -203,7 +423,7 @@ SpatCategories GetCategories(char **pCat, std::string name) {
 
 	std::vector<long> id(n);
 	std::iota(id.begin(), id.end(), 0);
-	scat.d.add_column(id, "ID");
+	scat.d.add_column(id, "value");
 
 	std::vector<std::string> nms(n);
 	for (size_t i = 0; i<n; i++) {
@@ -241,7 +461,7 @@ std::string basename_sds(std::string f) {
 	}
 	f.erase(std::remove(f.begin(), f.end(), '"'), f.end());
 
-/*	
+/*
 	f = std::regex_replace(f, std::regex("\\.h5$"), "");
 	f = std::regex_replace(f, std::regex("\\.hdf$"), "");
 	f = std::regex_replace(f, std::regex("\\.nc$"), "");
@@ -328,7 +548,7 @@ SpatRasterStack::SpatRasterStack(std::string fname, std::vector<int> ids, bool u
 	if (metadata == NULL) {
 		setError("file has no subdatasets");
 		GDALClose( (GDALDatasetH) poDataset );
-		return;	
+		return;
 	}
 
 	std::vector<std::string> meta; 
@@ -378,10 +598,10 @@ SpatRaster SpatRaster::fromFiles(std::vector<std::string> fname, std::vector<int
 		SpatRaster r;
 		bool ok = r.constructFromFile(fname[i], subds, subdsname, options);
 		if (r.msg.has_warning) {
-			out.addWarning(r.msg.warnings[0]);	
+			out.addWarning(r.msg.warnings[0]);
 		}
 		if (ok) {
-			out.addSource(r);
+			out.addSource(r, false);
 			if (r.msg.has_error) {
 				out.setError(r.msg.error);
 				return out;
@@ -414,19 +634,17 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	int nl = poDataset->GetRasterCount();
 	std::string gdrv = poDataset->GetDriver()->GetDescription();
 
-	if (nl == 0) {
+	char **metasds = poDataset->GetMetadata("SUBDATASETS");
+	if (metasds != NULL) {
 		std::vector<std::string> meta;
-		char **metadata = poDataset->GetMetadata("SUBDATASETS");
-		if (metadata != NULL) {
-			for (size_t i=0; metadata[i] != NULL; i++) {
-				meta.push_back(metadata[i]);
-			}
-			GDALClose( (GDALDatasetH) poDataset );
-			return constructFromSDS(fname, meta, subds, subdsname, options, gdrv=="netCDF"); 
-		} else {
-			setError("no data detected in " + fname);
-			return false;
+		for (size_t i=0; metasds[i] != NULL; i++) {
+			meta.push_back(metasds[i]);
 		}
+		GDALClose( (GDALDatasetH) poDataset );
+		return constructFromSDS(fname, meta, subds, subdsname, options, gdrv=="netCDF"); 
+	} else if (nl==0) {
+		setError("no raster data in " + fname);
+		return false;
 	}
 
 
@@ -445,7 +663,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	if( poDataset->GetGeoTransform( adfGeoTransform ) == CE_None ) {
 
 		double xmin = adfGeoTransform[0]; /* left x */
-		double xmax = xmin + adfGeoTransform[1] * s.ncol; /* w-e pixel resolution */
+		double xmax = xmin + adfGeoTransform[1] * s.ncol; /* w-e resolution */
 		//xmax = roundn(xmax, 9);
 		double ymax = adfGeoTransform[3]; // top y 
 		double ymin = ymax + s.nrow * adfGeoTransform[5]; 
@@ -458,7 +676,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 
 		SpatExtent e(xmin, xmax, ymin, ymax);
 		s.extent = e;
-	
+
 		if (adfGeoTransform[2] != 0 || adfGeoTransform[4] != 0) {
 			s.rotated = true;
 			addWarning("the data in this file are rotated. Use 'rectify' to fix that");
@@ -506,6 +724,27 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 		addWarning(msg);
 	}
 
+	std::vector<int_64> timestamps;
+	std::string timestep="raw";
+	std::vector<std::string> units;
+
+	try {
+		read_aux_json(fname, timestamps, timestep, units, s.nlyr);
+	} catch(...) {
+		timestamps.resize(0);
+		units.resize(0);
+		addWarning("could not parse aux.json");
+	}
+	if (timestamps.size() > 0) {
+		s.time = timestamps;
+		s.timestep = timestep;
+		s.hasTime = true;
+	}
+	if (units.size() > 0) {
+		s.unit = units;
+		s.hasUnit = true;
+	}
+
 	GDALRasterBand  *poBand;
 	//int nBlockXSize, nBlockYSize;
 	double adfMinMax[2];
@@ -514,9 +753,8 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 //	s.layers.resize(1);
 //	std::string unit = "";
 
-	std::string varname = basename_noext(fname).substr(0,3);
+	s.source_name = basename_noext(fname);
 	std::vector<std::vector<std::string>> bandmeta(s.nlyr);
-
 	bool getCols = s.nlyr == 3;
 	std::vector<unsigned> rgb_lyrs(3, -99);
 
@@ -549,11 +787,11 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 			s.has_scale_offset[i] = true;
 		} 
 
-	
+
 		poBand->GetBlockSize(&bs1, &bs2);
 		s.blockcols[i] = bs1;
 		s.blockrows[i] = bs2;
-		
+
 		//std::string dtype = GDALGetDataTypeName(poBand->GetRasterDataType());
 
 		adfMinMax[0] = poBand->GetMinimum( &bGotMin );
@@ -589,21 +827,27 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 			s.hasCategories[i] = true;
 		} 
 
+		bool have_rat = false;
+		SpatCategories crat;
+
 		if (!s.hasCategories[i]) {
 			GDALRasterAttributeTable *rat = poBand->GetDefaultRAT();
 			if( rat != NULL ) {
-				SpatCategories catg = GetRAT(rat);
-				if (catg.d.nrow() > 0) {
+				crat = GetRAT(rat);
+				crat.vat = true;
+				if (crat.d.nrow() > 0) {
 					if (gdrv == "AIG") {
-						std::vector<std::string> catnms = catg.d.get_names();
+						std::vector<std::string> catnms = crat.d.get_names();
 						std::vector<std::string> compnms = {"ID", "VALUE", "COUNT"};
 						if ((catnms.size() > 3) || (catnms != compnms)) {
-							s.cats[i] = catg;
+							s.cats[i] = crat;
 							s.hasCategories[i] = true;
-						}	
+							have_rat = true;
+						}
 					} else {
-						s.cats[i] = catg;
+						s.cats[i] = crat;
 						s.hasCategories[i] = true;
+						have_rat = true;
 					}
 				}
 			}
@@ -612,12 +856,19 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 		//		s.cats[i].d.cbind(crat.d); // needs more checking.
 		//	} else {
 
-
 		if (!s.hasCategories[i]) {
-			SpatCategories vat;
-			if (GetVAT(fname, vat)) {
-				s.cats[i] = vat;
-				s.hasCategories[i] = true;				
+			if (GetVAT(fname, crat)) {
+				s.cats[i] = crat;
+				s.hasCategories[i] = true;
+				have_rat = true;
+			}
+		}
+
+		if ((!s.hasColors[i]) && (have_rat)) {
+			SpatDataFrame ratcols;
+			if (colsFromRat(crat.d, ratcols)) {
+				s.hasColors[i] = true;
+				s.cols[i] = ratcols;
 			}
 		}
 
@@ -632,11 +883,16 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 			if (bandname != "") {
 				nm = bandname;
 			} else if (s.nlyr > 1) {
-				nm = varname + "_" + std::to_string(i+1);
+				nm = s.source_name + "_" + std::to_string(i+1);
 			} else {
 				nm = basename_noext(fname) ;
 			}
 		}
+
+		std::string dtype = GDALGetDataTypeName(poBand->GetRasterDataType());
+		if (in_string(dtype, "Int") || (dtype == "Byte")) {
+			s.valueType[i] = 1;
+		}
 		s.names[i] = nm;
 	}
 
@@ -659,7 +915,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	setSource(s);
 
 	if (getCols) {
-		setRGB(rgb_lyrs[0], rgb_lyrs[1], rgb_lyrs[2], -99);
+		setRGB(rgb_lyrs[0], rgb_lyrs[1], rgb_lyrs[2], -99, "rgb");
 	}
 
 	return true;
@@ -667,7 +923,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 
 
 bool SpatRaster::readStartGDAL(unsigned src) {
-    GDALDataset *poDataset = openGDAL(source[src].filename, GDAL_OF_RASTER | GDAL_OF_READONLY, source[src].open_ops);	
+    GDALDataset *poDataset = openGDAL(source[src].filename, GDAL_OF_RASTER | GDAL_OF_READONLY, source[src].open_ops);
 	if( poDataset == NULL )  {
 		setError("cannot read from " + source[src].filename );
 		return false;
@@ -714,7 +970,7 @@ void NAso(std::vector<double> &d, size_t n, const std::vector<double> &flags, co
 		}
 	}
 	if (haveUserNAflag) {
-		std::replace(d.begin(), d.end(), userNAflag, na); 		
+		std::replace(d.begin(), d.end(), userNAflag, na); 
 	}
 }
 
@@ -775,16 +1031,16 @@ void SpatRaster::readChunkGDAL(std::vector<double> &data, unsigned src, size_t r
 	if (panBandMap.size() > 0) {
 		err = source[src].gdalconnection->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], ncols, nrows, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
 	} else {
-		err = source[src].gdalconnection->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], ncols, nrows, GDT_Float64, nl, NULL, 0, 0, 0, NULL);	
+		err = source[src].gdalconnection->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], ncols, nrows, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
 	}
-	
+
 	GDALRasterBand  *poBand;
 	if (err == CE_None ) { 
 		for (size_t i=0; i<nl; i++) {
 			poBand = source[src].gdalconnection->GetRasterBand(source[src].layers[i]+1);
 			double naflag = poBand->GetNoDataValue(&hasNA);
 			if (hasNA)  naflags[i] = naflag;
-		}		
+		}
 		NAso(out, ncell, naflags, source[src].scale, source[src].offset, source[src].has_scale_offset, source[src].hasNAflag, source[src].NAflag);
 	}
 
@@ -810,7 +1066,7 @@ void SpatRaster::readChunkGDAL(std::vector<double> &data, unsigned src, size_t r
 	if (source[src].flipped) {
 		vflip(out, ncell, nrows, ncols, nl);
 	}
-	data.insert(data.end(), out.begin(), out.end());		
+	data.insert(data.end(), out.begin(), out.end());
 }
 
 
@@ -831,7 +1087,7 @@ std::vector<double> SpatRaster::readValuesGDAL(unsigned src, size_t row, size_t
 
     GDALDataset *poDataset = openGDAL(source[src].filename, GDAL_OF_RASTER | GDAL_OF_READONLY, source[src].open_ops);
 	GDALRasterBand *poBand;
-	
+
     if( poDataset == NULL )  {
 		setError("cannot read values. Does the file still exist?");
 		return errout;
@@ -921,7 +1177,7 @@ std::vector<double> SpatRaster::readGDALsample(unsigned src, size_t srows, size_
 			panBandMap.push_back(source[src].layers[i]+1);
 		}
 	}
-/*	
+/*
 	if (panBandMap.size() > 0) {
 		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], scols, srows, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
 	} else {
@@ -932,9 +1188,9 @@ std::vector<double> SpatRaster::readGDALsample(unsigned src, size_t srows, size_
 	if (panBandMap.size() > 0) {
 		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], scols, srows, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
 	} else {
-		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], scols, srows, GDT_Float64, nl, NULL, 0, 0, 0, NULL);	
+		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], scols, srows, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
 	}
-	
+
 	if (err == CE_None ) { 
 		GDALRasterBand *poBand;
 		for (size_t i=0; i<nl; i++) {
@@ -949,7 +1205,7 @@ std::vector<double> SpatRaster::readGDALsample(unsigned src, size_t srows, size_
 /*
 	for (size_t i=0; i < nl; i++) {
 		poBand = poDataset->GetRasterBand(source[src].layers[i] + 1);
-		size_t off = i * ncell;	
+		size_t off = i * ncell;
 		err = poBand->RasterIO(GF_Read, 0, 0, ncol(), nrow(), &out[off], scols, srows, GDT_Float64, 0, 0);
 		if (err != CE_None ) { break; }
 		double naflag = poBand->GetNoDataValue(&hasNA);
@@ -984,7 +1240,7 @@ std::vector<std::vector<double>> SpatRaster::readRowColGDAL(unsigned src, std::v
     GDALDataset *poDataset = openGDAL(source[src].filename, GDAL_OF_RASTER | GDAL_OF_READONLY, source[src].open_ops);
 
 	GDALRasterBand *poBand;
-	
+
     if( poDataset == NULL )  {
 		return errout;
 	}
@@ -1065,7 +1321,7 @@ std::vector<double> SpatRaster::readRowColGDALFlat(unsigned src, std::vector<int
     GDALDataset *poDataset = openGDAL(source[src].filename, GDAL_OF_RASTER | GDAL_OF_READONLY, source[src].open_ops);
 
 	GDALRasterBand *poBand;
-	
+
     if( poDataset == NULL )  {
 		return errout;
 	}
@@ -1177,7 +1433,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 	std::vector<std::vector<std::string>> info = parse_metadata_sds(meta);
 	int n = info[0].size();
 	std::vector<std::string> sd, varname, srcname;
-	
+
 // std::vector<unsigned> varnl;
 // for selection based on nlyr
 
@@ -1236,7 +1492,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 					srcname.push_back(info[2][i]);
 					rows.push_back(stoi(info[3][i]));
 					cols.push_back(stoi(info[4][i]));
-				}		
+				}
 			}
 		}
 		// pick the ones with most rows and then cols
@@ -1247,14 +1503,15 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 
 	std::vector<size_t> srcnl;
 	size_t cnt;
+	
     for (cnt=0; cnt < sd.size(); cnt++) {
 		if (constructFromFile(sd[cnt], {-1}, {""}, options)) break;
 	}
 //	source[0].source_name = srcname[cnt];
-	
+
 	std::vector<std::string> skipped, used;
 	srcnl.push_back(nlyr());
-	used.push_back(varname[0]);			
+	used.push_back(varname[0]);
 	SpatRaster out;
     for (size_t i=(cnt+1); i < sd.size(); i++) {
 //		printf( "%s\n", sd[i].c_str() );
@@ -1262,9 +1519,9 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 		if (success) {
 			if (out.compare_geom(*this, false, false, 0.1)) {
 //				out.source	[0].source_name = srcname[i];
-				addSource(out);
+				addSource(out, false);
 				srcnl.push_back(out.nlyr());
-				used.push_back(varname[i]);			
+				used.push_back(varname[i]);
 			} else {
 				skipped.push_back(varname[i]);
 			}
@@ -1317,19 +1574,17 @@ bool get_long(std::string input, long &output) {
     try  {
 		output = std::stoi(input);
 		return true;
-    }
-    catch (std::invalid_argument &e)  {
+    } catch (std::invalid_argument &e)  {
 		return false;
     }
 }
 
 
 bool get_double(std::string input, double &output) {
-    try  {
+    try {
 		output = std::stod(input);
 		return true;
-    }
-    catch (std::invalid_argument &e)  {
+    } catch (std::invalid_argument &e)  {
 		return false;
     }
 }
@@ -1393,7 +1648,7 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 		} else if ((origin.find("days")) != std::string::npos) {
 			days = true;
 		} else if ((origin.find("seconds")) != std::string::npos) {
-			seconds = true;	
+			seconds = true;
 		} 
 		size_t pos;
 		if ((pos = origin.find("from")) != std::string::npos) {
@@ -1430,7 +1685,7 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 			offset = get_time_string(origin);
 			for (size_t i=0; i<raw.size(); i++) out.push_back(raw[i]+offset);
 		} else {
-			step = "raw";			
+			step = "raw";
 			for (size_t i=0; i<raw.size(); i++) out.push_back(raw[i]);
 		}
 	}
@@ -1461,12 +1716,12 @@ std::vector<std::vector<std::string>> ncdf_names(const std::vector<std::vector<s
 		if (pos != std::string::npos) {
 			units = b[j].erase(0, pos+6);
 			continue;
-		}	
+		}
 		pos = b[j].find("long_name=");
 		if (pos != std::string::npos) {
 			lname = b[j].erase(0, pos+10);
 			continue;
-		}	
+		}
 		pos = b[j].find("standard_name=");
 		if (pos != std::string::npos) {
 			if (lname == "") {
@@ -1510,11 +1765,11 @@ void SpatRasterSource::set_names_time_ncdf(std::vector<std::string> metadata, st
 	}
 	source_name = nms[2][0];
 	source_name_long = nms[2][1];
-	
+
 	if (nms[2][2].size() == 0) {
 		unit = {""};
 	} else {
-		unit = {nms[2][2]};		
+		unit = {nms[2][2]};
 	}
 
 	recycle(unit, nlyr);
@@ -1523,7 +1778,7 @@ void SpatRasterSource::set_names_time_ncdf(std::vector<std::string> metadata, st
 	if (nms[0].size() > 0) {
 		std::string step;
 		std::vector<int_64> x = ncdf_time(metadata, nms[0], step, msg);
-	
+
 		if (x.size() == nlyr) {
 			time = x;
 			timestep = step;
