@@ -21,6 +21,8 @@
 // A collection of (perhaps non matching) SpatRasters 
 class SpatRasterCollection {
 	public:
+		virtual ~SpatRasterCollection(){}
+	
 		SpatMessages msg;
 		void setError(std::string s) { msg.setError(s); }
 		void addWarning(std::string s) { msg.addWarning(s); }
@@ -41,6 +43,7 @@ class SpatRasterCollection {
 			}
 		}
 		SpatRaster merge(SpatOptions &opt);
+		SpatRaster morph(SpatRaster &x, SpatOptions &opt);
 		SpatRaster mosaic(std::string fun, SpatOptions &opt);
 		SpatRaster summary(std::string fun, SpatOptions &opt);
 		
@@ -49,6 +52,8 @@ class SpatRasterCollection {
 // A class for "sub-datasets" 
 class SpatRasterStack {
 	public:
+		virtual ~SpatRasterStack(){}
+	
 		SpatMessages msg;
 		void setError(std::string s) { msg.setError(s); }
 		void addWarning(std::string s) { msg.addWarning(s); }
@@ -160,6 +165,16 @@ class SpatRasterStack {
 			names.push_back(name);
 			long_names.push_back(longname);
 			units.push_back(unit);
+			if (r.hasWarning()) {
+				for (size_t i=0; i<r.msg.warnings.size(); i++) {
+					addWarning(r.msg.warnings[i]);
+				}
+			}
+			if (r.hasError()) {
+				setError(r.msg.getError());
+				return false;
+			}
+			
 			return true;
 		};
 		
@@ -200,6 +215,22 @@ class SpatRasterStack {
 			} 
 			return out;
 		}
+
+		SpatRasterStack crop(SpatExtent e, std::string snap, SpatOptions &opt) {
+			SpatRasterStack out;
+			std::vector<std::string> ff = opt.get_filenames();
+			if (ff.size() != ds.size()) {
+				opt.set_filenames({""});
+				opt.ncopies *= ds.size();
+			}
+			for (size_t i=0; i<ds.size(); i++) {
+				out.push_back(ds[i].crop(e, snap, opt), names[i], long_names[i], units[i], true);
+				if (has_error()) {
+					return(out);
+				}
+			}
+			return out;
+		}
 		
 		void replace(unsigned i, SpatRaster x) {
 			if (i > (ds.size()-1)) {
